from cargo2rpm.rpm import _requires_crate, _requires_feature
from cargo2rpm.utils import load_metadata_from_resource, short_repr

import pytest


@pytest.mark.parametrize(
    "filename,expected",
    [
        (
            "ahash-0.8.3.json",
            {
                "cargo",
                "(crate(cfg-if/default) >= 1.0.0 with crate(cfg-if/default) < 2.0.0~)",
                "(crate(once_cell) >= 1.13.1 with crate(once_cell) < 2.0.0~)",
                "(crate(once_cell/unstable) >= 1.13.1 with crate(once_cell/unstable) < 2.0.0~)",
                "(crate(once_cell/alloc) >= 1.13.1 with crate(once_cell/alloc) < 2.0.0~)",
                "(crate(version_check/default) >= 0.9.4 with crate(version_check/default) < 0.10.0~)",
            },
        ),
        (
            "assert_cmd-2.0.8.json",
            {
                "cargo",
                "rust >= 1.60.0",
                "(crate(bstr/default) >= 1.0.1 with crate(bstr/default) < 2.0.0~)",
                "(crate(doc-comment/default) >= 0.3.0 with crate(doc-comment/default) < 0.4.0~)",
                "(crate(predicates) >= 2.1.0 with crate(predicates) < 3.0.0~)",
                "(crate(predicates/diff) >= 2.1.0 with crate(predicates/diff) < 3.0.0~)",
                "(crate(predicates-core/default) >= 1.0.0 with crate(predicates-core/default) < 2.0.0~)",
                "(crate(predicates-tree/default) >= 1.0.0 with crate(predicates-tree/default) < 2.0.0~)",
                "(crate(wait-timeout/default) >= 0.2.0 with crate(wait-timeout/default) < 0.3.0~)",
            },
        ),
        (
            "assert_fs-1.0.10.json",
            {
                "cargo",
                "rust >= 1.60.0",
                "(crate(doc-comment/default) >= 0.3.0 with crate(doc-comment/default) < 0.4.0~)",
                "(crate(globwalk/default) >= 0.8.0 with crate(globwalk/default) < 0.9.0~)",
                "(crate(predicates) >= 2.0.3 with crate(predicates) < 3.0.0~)",
                "(crate(predicates/diff) >= 2.0.3 with crate(predicates/diff) < 3.0.0~)",
                "(crate(predicates-core/default) >= 1.0.0 with crate(predicates-core/default) < 2.0.0~)",
                "(crate(predicates-tree/default) >= 1.0.0 with crate(predicates-tree/default) < 2.0.0~)",
                "(crate(tempfile/default) >= 3.0.0 with crate(tempfile/default) < 4.0.0~)",
            },
        ),
        (
            "autocfg-1.1.0.json",
            {
                "cargo",
            },
        ),
        (
            "bstr-1.2.0.json",
            {
                "cargo",
                "rust >= 1.60",
                "(crate(memchr) >= 2.4.0 with crate(memchr) < 3.0.0~)",
            },
        ),
        (
            "cfg-if-1.0.0.json",
            {
                "cargo",
            },
        ),
        (
            "clap-4.1.4.json",
            {
                "cargo",
                "rust >= 1.64.0",
                "(crate(bitflags/default) >= 1.2.0 with crate(bitflags/default) < 2.0.0~)",
                "(crate(clap_lex/default) >= 0.3.0 with crate(clap_lex/default) < 0.4.0~)",
            },
        ),
        (
            "gstreamer-0.19.7.json",
            {
                "cargo",
                "rust >= 1.63",
                "(crate(bitflags/default) >= 1.0.0 with crate(bitflags/default) < 2.0.0~)",
                "(crate(cfg-if/default) >= 1.0.0 with crate(cfg-if/default) < 2.0.0~)",
                "(crate(gstreamer-sys/default) >= 0.19.0 with crate(gstreamer-sys/default) < 0.20.0~)",
                "(crate(futures-channel/default) >= 0.3.0 with crate(futures-channel/default) < 0.4.0~)",
                "(crate(futures-core/default) >= 0.3.0 with crate(futures-core/default) < 0.4.0~)",
                "(crate(futures-util) >= 0.3.0 with crate(futures-util) < 0.4.0~)",
                "(crate(glib/default) >= 0.16.2 with crate(glib/default) < 0.17.0~)",
                "(crate(libc/default) >= 0.2.0 with crate(libc/default) < 0.3.0~)",
                "(crate(muldiv/default) >= 1.0.0 with crate(muldiv/default) < 2.0.0~)",
                "(crate(num-integer) >= 0.1.0 with crate(num-integer) < 0.2.0~)",
                "(crate(num-rational) >= 0.4.0 with crate(num-rational) < 0.5.0~)",
                "(crate(once_cell/default) >= 1.0.0 with crate(once_cell/default) < 2.0.0~)",
                "(crate(option-operations/default) >= 0.5.0 with crate(option-operations/default) < 0.6.0~)",
                "(crate(paste/default) >= 1.0.0 with crate(paste/default) < 2.0.0~)",
                "(crate(pretty-hex/default) >= 0.3.0 with crate(pretty-hex/default) < 0.4.0~)",
                "(crate(thiserror/default) >= 1.0.0 with crate(thiserror/default) < 2.0.0~)",
            },
        ),
        (
            "human-panic-1.1.0.json",
            {
                "cargo",
                "rust >= 1.60.0",
                "(crate(backtrace/default) >= 0.3.9 with crate(backtrace/default) < 0.4.0~)",
                "(crate(os_info/default) >= 2.0.6 with crate(os_info/default) < 3.0.0~)",
                "(crate(serde/default) >= 1.0.79 with crate(serde/default) < 2.0.0~)",
                "(crate(serde_derive/default) >= 1.0.79 with crate(serde_derive/default) < 2.0.0~)",
                "(crate(toml/default) >= 0.5.0 with crate(toml/default) < 0.6.0~)",
                "(crate(uuid) >= 0.8.0 with crate(uuid) < 0.9.0~)",
                "(crate(uuid/v4) >= 0.8.0 with crate(uuid/v4) < 0.9.0~)",
            },
        ),
        (
            "libc-0.2.139.json",
            {
                "cargo",
            },
        ),
        (
            "predicates-2.1.5.json",
            {
                "cargo",
                "rust >= 1.60.0",
                "(crate(itertools/default) >= 0.10.0 with crate(itertools/default) < 0.11.0~)",
                "(crate(predicates-core/default) >= 1.0.0 with crate(predicates-core/default) < 2.0.0~)",
            },
        ),
        (
            "proc-macro2-1.0.50.json",
            {
                "cargo",
                "rust >= 1.31",
                "(crate(unicode-ident/default) >= 1.0.0 with crate(unicode-ident/default) < 2.0.0~)",
            },
        ),
        (
            "quote-1.0.23.json",
            {
                "cargo",
                "rust >= 1.31",
                "(crate(proc-macro2) >= 1.0.40 with crate(proc-macro2) < 2.0.0~)",
            },
        ),
        (
            "rand-0.8.5.json",
            {
                "cargo",
                "(crate(rand_core/default) >= 0.6.0 with crate(rand_core/default) < 0.7.0~)",
            },
        ),
        (
            "rand_core-0.6.4.json",
            {
                "cargo",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            {
                "cargo",
                "rust >= 1.60",
                "(crate(arrayvec) >= 0.7.0 with crate(arrayvec) < 0.8.0~)",
                "(crate(num-traits) >= 0.2.0 with crate(num-traits) < 0.3.0~)",
                "(crate(num-traits/i128) >= 0.2.0 with crate(num-traits/i128) < 0.3.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            {
                "cargo",
                "rust >= 1.48",
                "(crate(bitflags/default) >= 1.2.1 with crate(bitflags/default) < 2.0.0~)",
                "(crate(linux-raw-sys) >= 0.1.2 with crate(linux-raw-sys) < 0.2.0~)",
                "(crate(linux-raw-sys/errno) >= 0.1.2 with crate(linux-raw-sys/errno) < 0.2.0~)",
                "(crate(linux-raw-sys/general) >= 0.1.2 with crate(linux-raw-sys/general) < 0.2.0~)",
                "(crate(linux-raw-sys/ioctl) >= 0.1.2 with crate(linux-raw-sys/ioctl) < 0.2.0~)",
                "(crate(linux-raw-sys/no_std) >= 0.1.2 with crate(linux-raw-sys/no_std) < 0.2.0~)",
                "(crate(libc/default) >= 0.2.133 with crate(libc/default) < 0.3.0~)",
                "(crate(libc/extra_traits) >= 0.2.133 with crate(libc/extra_traits) < 0.3.0~)",
                "(crate(errno) >= 0.2.8 with crate(errno) < 0.3.0~)",
                "(crate(windows-sys/default) >= 0.45.0 with crate(windows-sys/default) < 0.46.0~)",
                "(crate(windows-sys/Win32_Foundation) >= 0.45.0 with crate(windows-sys/Win32_Foundation) < 0.46.0~)",
                "(crate(windows-sys/Win32_Networking_WinSock) >= 0.45.0 with crate(windows-sys/Win32_Networking_WinSock) < 0.46.0~)",
                "(crate(windows-sys/Win32_NetworkManagement_IpHelper) >= 0.45.0 with crate(windows-sys/Win32_NetworkManagement_IpHelper) < 0.46.0~)",
                "(crate(windows-sys/Win32_System_Threading) >= 0.45.0 with crate(windows-sys/Win32_System_Threading) < 0.46.0~)",
            },
        ),
        (
            "serde-1.0.152.json",
            {
                "cargo",
                "rust >= 1.13",
            },
        ),
        (
            "serde_derive-1.0.152.json",
            {
                "cargo",
                "rust >= 1.31",
                "(crate(proc-macro2/default) >= 1.0.0 with crate(proc-macro2/default) < 2.0.0~)",
                "(crate(quote/default) >= 1.0.0 with crate(quote/default) < 2.0.0~)",
                "(crate(syn/default) >= 1.0.104 with crate(syn/default) < 2.0.0~)",
            },
        ),
        (
            "sha1collisiondetection-0.3.1.json",
            {
                "cargo",
                "rust >= 1.60",
                "(crate(generic-array/default) >= 0.12.0 with crate(generic-array/default) < 0.15.0~)",
            },
        ),
        (
            "syn-1.0.107.json",
            {
                "cargo",
                "rust >= 1.31",
                "(crate(proc-macro2) >= 1.0.46 with crate(proc-macro2) < 2.0.0~)",
                "(crate(unicode-ident/default) >= 1.0.0 with crate(unicode-ident/default) < 2.0.0~)",
            },
        ),
        (
            "time-0.3.17.json",
            {
                "cargo",
                "rust >= 1.60.0",
                "crate(time-core/default) = 0.1.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            {
                "cargo",
                "rust >= 1.49",
                "(crate(pin-project-lite/default) >= 0.2.0 with crate(pin-project-lite/default) < 0.3.0~)",
                "(crate(autocfg/default) >= 1.1.0 with crate(autocfg/default) < 2.0.0~)",
                "(crate(windows-sys/default) >= 0.42.0 with crate(windows-sys/default) < 0.43.0~)",
                "(crate(windows-sys/Win32_Foundation) >= 0.42.0 with crate(windows-sys/Win32_Foundation) < 0.43.0~)",
                "(crate(windows-sys/Win32_Security_Authorization) >= 0.42.0 with crate(windows-sys/Win32_Security_Authorization) < 0.43.0~)",
            },
        ),
        (
            "unicode-xid-0.2.4.json",
            {
                "cargo",
                "rust >= 1.17",
            },
        ),
        (
            "zbus-3.8.0.json",
            {
                "cargo",
                "rust >= 1.60",
                "(crate(async-broadcast/default) >= 0.5.0 with crate(async-broadcast/default) < 0.6.0~)",
                "(crate(async-recursion/default) >= 1.0.0 with crate(async-recursion/default) < 2.0.0~)",
                "(crate(async-trait/default) >= 0.1.58 with crate(async-trait/default) < 0.2.0~)",
                "(crate(byteorder/default) >= 1.4.3 with crate(byteorder/default) < 2.0.0~)",
                "(crate(derivative/default) >= 2.2.0 with crate(derivative/default) < 3.0.0~)",
                "(crate(dirs/default) >= 4.0.0 with crate(dirs/default) < 5.0.0~)",
                "(crate(enumflags2/default) >= 0.7.5 with crate(enumflags2/default) < 0.8.0~)",
                "(crate(enumflags2/serde) >= 0.7.5 with crate(enumflags2/serde) < 0.8.0~)",
                "(crate(event-listener/default) >= 2.5.3 with crate(event-listener/default) < 3.0.0~)",
                "(crate(futures-core/default) >= 0.3.25 with crate(futures-core/default) < 0.4.0~)",
                "(crate(futures-sink/default) >= 0.3.25 with crate(futures-sink/default) < 0.4.0~)",
                "(crate(futures-util) >= 0.3.25 with crate(futures-util) < 0.4.0~)",
                "(crate(futures-util/sink) >= 0.3.25 with crate(futures-util/sink) < 0.4.0~)",
                "(crate(futures-util/std) >= 0.3.25 with crate(futures-util/std) < 0.4.0~)",
                "(crate(hex/default) >= 0.4.3 with crate(hex/default) < 0.5.0~)",
                "(crate(nix/default) >= 0.25.0 with crate(nix/default) < 0.26.0~)",
                "(crate(once_cell/default) >= 1.4.0 with crate(once_cell/default) < 2.0.0~)",
                "(crate(ordered-stream/default) >= 0.1.4 with crate(ordered-stream/default) < 0.2.0~)",
                "(crate(rand/default) >= 0.8.5 with crate(rand/default) < 0.9.0~)",
                "(crate(serde/default) >= 1.0.0 with crate(serde/default) < 2.0.0~)",
                "(crate(serde/derive) >= 1.0.0 with crate(serde/derive) < 2.0.0~)",
                "(crate(serde_repr/default) >= 0.1.9 with crate(serde_repr/default) < 0.2.0~)",
                "(crate(sha1/default) >= 0.10.5 with crate(sha1/default) < 0.11.0~)",
                "(crate(sha1/std) >= 0.10.5 with crate(sha1/std) < 0.11.0~)",
                "(crate(static_assertions/default) >= 1.1.0 with crate(static_assertions/default) < 2.0.0~)",
                "(crate(tracing/default) >= 0.1.37 with crate(tracing/default) < 0.2.0~)",
                "crate(zbus_macros/default) = 3.8.0",
                "(crate(zbus_names/default) >= 2.5.0 with crate(zbus_names/default) < 3.0.0~)",
                "(crate(zvariant) >= 3.10.0 with crate(zvariant) < 4.0.0~)",
                "(crate(zvariant/enumflags2) >= 3.10.0 with crate(zvariant/enumflags2) < 4.0.0~)",
                "(crate(uds_windows/default) >= 1.0.2 with crate(uds_windows/default) < 2.0.0~)",
                "(crate(winapi/default) >= 0.3.0 with crate(winapi/default) < 0.4.0~)",
                "(crate(winapi/handleapi) >= 0.3.0 with crate(winapi/handleapi) < 0.4.0~)",
                "(crate(winapi/iphlpapi) >= 0.3.0 with crate(winapi/iphlpapi) < 0.4.0~)",
                "(crate(winapi/memoryapi) >= 0.3.0 with crate(winapi/memoryapi) < 0.4.0~)",
                "(crate(winapi/processthreadsapi) >= 0.3.0 with crate(winapi/processthreadsapi) < 0.4.0~)",
                "(crate(winapi/sddl) >= 0.3.0 with crate(winapi/sddl) < 0.4.0~)",
                "(crate(winapi/securitybaseapi) >= 0.3.0 with crate(winapi/securitybaseapi) < 0.4.0~)",
                "(crate(winapi/synchapi) >= 0.3.0 with crate(winapi/synchapi) < 0.4.0~)",
                "(crate(winapi/tcpmib) >= 0.3.0 with crate(winapi/tcpmib) < 0.4.0~)",
                "(crate(winapi/winbase) >= 0.3.0 with crate(winapi/winbase) < 0.4.0~)",
                "(crate(winapi/winerror) >= 0.3.0 with crate(winapi/winerror) < 0.4.0~)",
                "(crate(winapi/winsock2) >= 0.3.0 with crate(winapi/winsock2) < 0.4.0~)",
            },
        ),
    ],
    ids=short_repr,
)
def test_requires_crate(filename: str, expected: set[str]):
    metadata = load_metadata_from_resource(filename)
    assert _requires_crate(metadata.packages[0]) == expected


@pytest.mark.parametrize(
    "filename,feature,expected",
    [
        (
            "ahash-0.8.3.json",
            "default",
            {
                "cargo",
                "crate(ahash) = 0.8.3",
                "crate(ahash/std) = 0.8.3",
                "crate(ahash/runtime-rng) = 0.8.3",
            },
        ),
        (
            "ahash-0.8.3.json",
            "atomic-polyfill",
            {
                "cargo",
                "crate(ahash) = 0.8.3",
                "(crate(atomic-polyfill/default) >= 1.0.1 with crate(atomic-polyfill/default) < 2.0.0~)",
                "(crate(once_cell/atomic-polyfill) >= 1.13.1 with crate(once_cell/atomic-polyfill) < 2.0.0~)",
            },
        ),
        (
            "ahash-0.8.3.json",
            "compile-time-rng",
            {
                "cargo",
                "crate(ahash) = 0.8.3",
                "crate(ahash/const-random) = 0.8.3",
            },
        ),
        (
            "ahash-0.8.3.json",
            "const-random",
            {
                "cargo",
                "crate(ahash) = 0.8.3",
                "(crate(const-random/default) >= 0.1.12 with crate(const-random/default) < 0.2.0~)",
            },
        ),
        (
            "ahash-0.8.3.json",
            "getrandom",
            {
                "cargo",
                "crate(ahash) = 0.8.3",
                "(crate(getrandom/default) >= 0.2.7 with crate(getrandom/default) < 0.3.0~)",
            },
        ),
        (
            "ahash-0.8.3.json",
            "no-rng",
            {
                "cargo",
                "crate(ahash) = 0.8.3",
            },
        ),
        (
            "ahash-0.8.3.json",
            "runtime-rng",
            {
                "cargo",
                "crate(ahash) = 0.8.3",
                "crate(ahash/getrandom) = 0.8.3",
            },
        ),
        (
            "ahash-0.8.3.json",
            "serde",
            {
                "cargo",
                "crate(ahash) = 0.8.3",
                "(crate(serde/default) >= 1.0.117 with crate(serde/default) < 2.0.0~)",
            },
        ),
        (
            "ahash-0.8.3.json",
            "std",
            {
                "cargo",
                "crate(ahash) = 0.8.3",
            },
        ),
        (
            "assert_cmd-2.0.8.json",
            "default",
            {
                "cargo",
                "crate(assert_cmd) = 2.0.8",
            },
        ),
        (
            "assert_cmd-2.0.8.json",
            "color",
            {
                "cargo",
                "crate(assert_cmd) = 2.0.8",
                "crate(concolor/default) = 0.0.11",
                "crate(concolor/std) = 0.0.11",
                "(crate(yansi/default) >= 0.5.1 with crate(yansi/default) < 0.6.0~)",
                "(crate(predicates/color) >= 2.1.0 with crate(predicates/color) < 3.0.0~)",
            },
        ),
        (
            "assert_cmd-2.0.8.json",
            "color-auto",
            {
                "cargo",
                "crate(assert_cmd) = 2.0.8",
                "crate(assert_cmd/color) = 2.0.8",
                "crate(concolor/auto) = 0.0.11",
                "crate(concolor/default) = 0.0.11",
            },
        ),
        (
            "assert_fs-1.0.10.json",
            "default",
            {
                "cargo",
                "crate(assert_fs) = 1.0.10",
            },
        ),
        (
            "assert_fs-1.0.10.json",
            "color",
            {
                "cargo",
                "crate(assert_fs) = 1.0.10",
                "crate(concolor/default) = 0.0.11",
                "(crate(yansi/default) >= 0.5.0 with crate(yansi/default) < 0.6.0~)",
                "(crate(predicates/color) >= 2.0.3 with crate(predicates/color) < 3.0.0~)",
            },
        ),
        (
            "assert_fs-1.0.10.json",
            "color-auto",
            {
                "cargo",
                "crate(assert_fs) = 1.0.10",
                "crate(assert_fs/color) = 1.0.10",
                "crate(concolor/auto) = 0.0.11",
                "crate(concolor/default) = 0.0.11",
            },
        ),
        (
            "autocfg-1.1.0.json",
            "default",
            {
                "cargo",
                "crate(autocfg) = 1.1.0",
            },
        ),
        (
            "bstr-1.2.0.json",
            "default",
            {
                "cargo",
                "crate(bstr) = 1.2.0",
                "crate(bstr/std) = 1.2.0",
                "crate(bstr/unicode) = 1.2.0",
            },
        ),
        (
            "bstr-1.2.0.json",
            "alloc",
            {
                "cargo",
                "crate(bstr) = 1.2.0",
                "(crate(serde) >= 1.0.85 with crate(serde) < 2.0.0~)",
                "(crate(serde/alloc) >= 1.0.85 with crate(serde/alloc) < 2.0.0~)",
            },
        ),
        (
            "bstr-1.2.0.json",
            "serde",
            {
                "cargo",
                "crate(bstr) = 1.2.0",
                "(crate(serde) >= 1.0.85 with crate(serde) < 2.0.0~)",
            },
        ),
        (
            "bstr-1.2.0.json",
            "std",
            {
                "cargo",
                "crate(bstr) = 1.2.0",
                "crate(bstr/alloc) = 1.2.0",
                "(crate(memchr/std) >= 2.4.0 with crate(memchr/std) < 3.0.0~)",
                "(crate(serde) >= 1.0.85 with crate(serde) < 2.0.0~)",
                "(crate(serde/std) >= 1.0.85 with crate(serde/std) < 2.0.0~)",
            },
        ),
        (
            "bstr-1.2.0.json",
            "unicode",
            {
                "cargo",
                "crate(bstr) = 1.2.0",
                "(crate(once_cell/default) >= 1.14.0 with crate(once_cell/default) < 2.0.0~)",
                "(crate(regex-automata) >= 0.1.5 with crate(regex-automata) < 0.2.0~)",
            },
        ),
        (
            "cfg-if-1.0.0.json",
            "default",
            {
                "cargo",
                "crate(cfg-if) = 1.0.0",
            },
        ),
        (
            "cfg-if-1.0.0.json",
            "compiler_builtins",
            {
                "cargo",
                "crate(cfg-if) = 1.0.0",
                "(crate(compiler_builtins/default) >= 0.1.2 with crate(compiler_builtins/default) < 0.2.0~)",
            },
        ),
        (
            "cfg-if-1.0.0.json",
            "core",
            {
                "cargo",
                "crate(cfg-if) = 1.0.0",
                "(crate(rustc-std-workspace-core/default) >= 1.0.0 with crate(rustc-std-workspace-core/default) < 2.0.0~)",
            },
        ),
        (
            "cfg-if-1.0.0.json",
            "rustc-dep-of-std",
            {
                "cargo",
                "crate(cfg-if) = 1.0.0",
                "crate(cfg-if/core) = 1.0.0",
                "crate(cfg-if/compiler_builtins) = 1.0.0",
            },
        ),
        (
            "clap-4.1.4.json",
            "default",
            {
                "cargo",
                "crate(clap) = 4.1.4",
                "crate(clap/std) = 4.1.4",
                "crate(clap/color) = 4.1.4",
                "crate(clap/help) = 4.1.4",
                "crate(clap/usage) = 4.1.4",
                "crate(clap/error-context) = 4.1.4",
                "crate(clap/suggestions) = 4.1.4",
            },
        ),
        (
            "clap-4.1.4.json",
            "cargo",
            {
                "cargo",
                "crate(clap) = 4.1.4",
                "(crate(once_cell/default) >= 1.12.0 with crate(once_cell/default) < 2.0.0~)",
            },
        ),
        (
            "clap-4.1.4.json",
            "color",
            {
                "cargo",
                "crate(clap) = 4.1.4",
                "(crate(is-terminal/default) >= 0.4.1 with crate(is-terminal/default) < 0.5.0~)",
                "(crate(termcolor/default) >= 1.1.1 with crate(termcolor/default) < 2.0.0~)",
            },
        ),
        (
            "clap-4.1.4.json",
            "debug",
            {
                "cargo",
                "crate(clap) = 4.1.4",
                "crate(clap_derive/debug) = 4.1.0",
                "crate(clap_derive/default) = 4.1.0",
                "(crate(backtrace/default) >= 0.3.0 with crate(backtrace/default) < 0.4.0~)",
            },
        ),
        (
            "clap-4.1.4.json",
            "deprecated",
            {
                "cargo",
                "crate(clap) = 4.1.4",
                "crate(clap_derive/default) = 4.1.0",
                "crate(clap_derive/deprecated) = 4.1.0",
            },
        ),
        (
            "clap-4.1.4.json",
            "derive",
            {
                "cargo",
                "crate(clap) = 4.1.4",
                "crate(clap_derive/default) = 4.1.0",
                "(crate(once_cell/default) >= 1.12.0 with crate(once_cell/default) < 2.0.0~)",
            },
        ),
        (
            "clap-4.1.4.json",
            "env",
            {
                "cargo",
                "crate(clap) = 4.1.4",
            },
        ),
        (
            "clap-4.1.4.json",
            "error-context",
            {
                "cargo",
                "crate(clap) = 4.1.4",
            },
        ),
        (
            "clap-4.1.4.json",
            "help",
            {
                "cargo",
                "crate(clap) = 4.1.4",
            },
        ),
        (
            "clap-4.1.4.json",
            "std",
            {
                "cargo",
                "crate(clap) = 4.1.4",
            },
        ),
        (
            "clap-4.1.4.json",
            "string",
            {
                "cargo",
                "crate(clap) = 4.1.4",
            },
        ),
        (
            "clap-4.1.4.json",
            "suggestions",
            {
                "cargo",
                "crate(clap) = 4.1.4",
                "crate(clap/error-context) = 4.1.4",
                "(crate(strsim/default) >= 0.10.0 with crate(strsim/default) < 0.11.0~)",
            },
        ),
        (
            "clap-4.1.4.json",
            "unicode",
            {
                "cargo",
                "crate(clap) = 4.1.4",
                "(crate(unicode-width/default) >= 0.1.9 with crate(unicode-width/default) < 0.2.0~)",
                "(crate(unicase/default) >= 2.6.0 with crate(unicase/default) < 3.0.0~)",
            },
        ),
        (
            "clap-4.1.4.json",
            "unstable-doc",
            {
                "cargo",
                "crate(clap) = 4.1.4",
                "crate(clap/derive) = 4.1.4",
                "crate(clap/cargo) = 4.1.4",
                "crate(clap/wrap_help) = 4.1.4",
                "crate(clap/env) = 4.1.4",
                "crate(clap/unicode) = 4.1.4",
                "crate(clap/string) = 4.1.4",
                "crate(clap/unstable-replace) = 4.1.4",
            },
        ),
        (
            "clap-4.1.4.json",
            "unstable-grouped",
            {
                "cargo",
                "crate(clap) = 4.1.4",
            },
        ),
        (
            "clap-4.1.4.json",
            "unstable-replace",
            {
                "cargo",
                "crate(clap) = 4.1.4",
            },
        ),
        (
            "clap-4.1.4.json",
            "unstable-v5",
            {
                "cargo",
                "crate(clap) = 4.1.4",
                "crate(clap/deprecated) = 4.1.4",
                "crate(clap_derive/default) = 4.1.0",
                "crate(clap_derive/unstable-v5) = 4.1.0",
            },
        ),
        (
            "clap-4.1.4.json",
            "usage",
            {
                "cargo",
                "crate(clap) = 4.1.4",
            },
        ),
        (
            "clap-4.1.4.json",
            "wrap_help",
            {
                "cargo",
                "crate(clap) = 4.1.4",
                "crate(clap/help) = 4.1.4",
                "(crate(terminal_size/default) >= 0.2.1 with crate(terminal_size/default) < 0.3.0~)",
            },
        ),
        (
            "gstreamer-0.19.7.json",
            "default",
            {
                "cargo",
                "crate(gstreamer) = 0.19.7",
            },
        ),
        (
            "gstreamer-0.19.7.json",
            "dox",
            {
                "cargo",
                "crate(gstreamer) = 0.19.7",
                "crate(gstreamer/serde) = 0.19.7",
                "(crate(glib/dox) >= 0.16.2 with crate(glib/dox) < 0.17.0~)",
                "(crate(gstreamer-sys/dox) >= 0.19.0 with crate(gstreamer-sys/dox) < 0.20.0~)",
            },
        ),
        (
            "gstreamer-0.19.7.json",
            "serde",
            {
                "cargo",
                "crate(gstreamer) = 0.19.7",
                "crate(gstreamer/serde_bytes) = 0.19.7",
                "(crate(num-rational/serde) >= 0.4.0 with crate(num-rational/serde) < 0.5.0~)",
                "(crate(serde/default) >= 1.0.0 with crate(serde/default) < 2.0.0~)",
                "(crate(serde/derive) >= 1.0.0 with crate(serde/derive) < 2.0.0~)",
            },
        ),
        (
            "gstreamer-0.19.7.json",
            "serde_bytes",
            {
                "cargo",
                "crate(gstreamer) = 0.19.7",
                "(crate(serde_bytes/default) >= 0.11.0 with crate(serde_bytes/default) < 0.12.0~)",
            },
        ),
        (
            "gstreamer-0.19.7.json",
            "v1_16",
            {
                "cargo",
                "crate(gstreamer) = 0.19.7",
                "(crate(gstreamer-sys/v1_16) >= 0.19.0 with crate(gstreamer-sys/v1_16) < 0.20.0~)",
            },
        ),
        (
            "gstreamer-0.19.7.json",
            "v1_18",
            {
                "cargo",
                "crate(gstreamer) = 0.19.7",
                "crate(gstreamer/v1_16) = 0.19.7",
                "(crate(gstreamer-sys/v1_18) >= 0.19.0 with crate(gstreamer-sys/v1_18) < 0.20.0~)",
            },
        ),
        (
            "gstreamer-0.19.7.json",
            "v1_20",
            {
                "cargo",
                "crate(gstreamer) = 0.19.7",
                "crate(gstreamer/v1_18) = 0.19.7",
                "(crate(gstreamer-sys/v1_20) >= 0.19.0 with crate(gstreamer-sys/v1_20) < 0.20.0~)",
            },
        ),
        (
            "gstreamer-0.19.7.json",
            "v1_22",
            {
                "cargo",
                "crate(gstreamer) = 0.19.7",
                "crate(gstreamer/v1_20) = 0.19.7",
                "(crate(gstreamer-sys/v1_22) >= 0.19.0 with crate(gstreamer-sys/v1_22) < 0.20.0~)",
            },
        ),
        (
            "human-panic-1.1.0.json",
            "default",
            {
                "cargo",
                "crate(human-panic) = 1.1.0",
                "crate(human-panic/color) = 1.1.0",
            },
        ),
        (
            "human-panic-1.1.0.json",
            "color",
            {
                "cargo",
                "crate(human-panic) = 1.1.0",
                "crate(concolor/default) = 0.0.11",
                "crate(concolor/auto) = 0.0.11",
                "(crate(termcolor/default) >= 1.0.4 with crate(termcolor/default) < 2.0.0~)",
            },
        ),
        (
            "human-panic-1.1.0.json",
            "nightly",
            {
                "cargo",
                "crate(human-panic) = 1.1.0",
            },
        ),
        (
            "libc-0.2.139.json",
            "default",
            {
                "cargo",
                "crate(libc) = 0.2.139",
                "crate(libc/std) = 0.2.139",
            },
        ),
        (
            "libc-0.2.139.json",
            "align",
            {
                "cargo",
                "crate(libc) = 0.2.139",
            },
        ),
        (
            "libc-0.2.139.json",
            "const-extern-fn",
            {
                "cargo",
                "crate(libc) = 0.2.139",
            },
        ),
        (
            "libc-0.2.139.json",
            "extra_traits",
            {
                "cargo",
                "crate(libc) = 0.2.139",
            },
        ),
        (
            "libc-0.2.139.json",
            "rustc-dep-of-std",
            {
                "cargo",
                "crate(libc) = 0.2.139",
                "crate(libc/align) = 0.2.139",
                "crate(libc/rustc-std-workspace-core) = 0.2.139",
            },
        ),
        (
            "libc-0.2.139.json",
            "rustc-std-workspace-core",
            {
                "cargo",
                "crate(libc) = 0.2.139",
                "(crate(rustc-std-workspace-core/default) >= 1.0.0 with crate(rustc-std-workspace-core/default) < 2.0.0~)",
            },
        ),
        (
            "libc-0.2.139.json",
            "std",
            {
                "cargo",
                "crate(libc) = 0.2.139",
            },
        ),
        (
            "libc-0.2.139.json",
            "use_std",
            {
                "cargo",
                "crate(libc) = 0.2.139",
                "crate(libc/std) = 0.2.139",
            },
        ),
        (
            "predicates-2.1.5.json",
            "default",
            {
                "cargo",
                "crate(predicates) = 2.1.5",
                "crate(predicates/diff) = 2.1.5",
                "crate(predicates/regex) = 2.1.5",
                "crate(predicates/float-cmp) = 2.1.5",
                "crate(predicates/normalize-line-endings) = 2.1.5",
            },
        ),
        (
            "predicates-2.1.5.json",
            "color",
            {
                "cargo",
                "crate(predicates) = 2.1.5",
                "crate(concolor/default) = 0.0.11",
                "crate(concolor/std) = 0.0.11",
                "(crate(yansi/default) >= 0.5.1 with crate(yansi/default) < 0.6.0~)",
            },
        ),
        (
            "predicates-2.1.5.json",
            "color-auto",
            {
                "cargo",
                "crate(predicates) = 2.1.5",
                "crate(predicates/color) = 2.1.5",
                "crate(concolor/auto) = 0.0.11",
                "crate(concolor/default) = 0.0.11",
            },
        ),
        (
            "predicates-2.1.5.json",
            "diff",
            {
                "cargo",
                "crate(predicates) = 2.1.5",
                "(crate(difflib/default) >= 0.4.0 with crate(difflib/default) < 0.5.0~)",
            },
        ),
        (
            "predicates-2.1.5.json",
            "float-cmp",
            {
                "cargo",
                "crate(predicates) = 2.1.5",
                "(crate(float-cmp/default) >= 0.9.0 with crate(float-cmp/default) < 0.10.0~)",
            },
        ),
        (
            "predicates-2.1.5.json",
            "normalize-line-endings",
            {
                "cargo",
                "crate(predicates) = 2.1.5",
                "(crate(normalize-line-endings/default) >= 0.3.0 with crate(normalize-line-endings/default) < 0.4.0~)",
            },
        ),
        (
            "predicates-2.1.5.json",
            "regex",
            {
                "cargo",
                "crate(predicates) = 2.1.5",
                "(crate(regex/default) >= 1.0.0 with crate(regex/default) < 2.0.0~)",
            },
        ),
        (
            "predicates-2.1.5.json",
            "unstable",
            {
                "cargo",
                "crate(predicates) = 2.1.5",
            },
        ),
        (
            "proc-macro2-1.0.50.json",
            "default",
            {
                "cargo",
                "crate(proc-macro2) = 1.0.50",
                "crate(proc-macro2/proc-macro) = 1.0.50",
            },
        ),
        (
            "proc-macro2-1.0.50.json",
            "nightly",
            {
                "cargo",
                "crate(proc-macro2) = 1.0.50",
            },
        ),
        (
            "proc-macro2-1.0.50.json",
            "proc-macro",
            {
                "cargo",
                "crate(proc-macro2) = 1.0.50",
            },
        ),
        (
            "proc-macro2-1.0.50.json",
            "span-locations",
            {
                "cargo",
                "crate(proc-macro2) = 1.0.50",
            },
        ),
        (
            "quote-1.0.23.json",
            "default",
            {
                "cargo",
                "crate(quote) = 1.0.23",
                "crate(quote/proc-macro) = 1.0.23",
            },
        ),
        (
            "quote-1.0.23.json",
            "proc-macro",
            {
                "cargo",
                "crate(quote) = 1.0.23",
                "(crate(proc-macro2/proc-macro) >= 1.0.40 with crate(proc-macro2/proc-macro) < 2.0.0~)",
            },
        ),
        (
            "rand-0.8.5.json",
            "default",
            {
                "cargo",
                "crate(rand) = 0.8.5",
                "crate(rand/std) = 0.8.5",
                "crate(rand/std_rng) = 0.8.5",
            },
        ),
        (
            "rand-0.8.5.json",
            "alloc",
            {
                "cargo",
                "crate(rand) = 0.8.5",
                "(crate(rand_core/alloc) >= 0.6.0 with crate(rand_core/alloc) < 0.7.0~)",
            },
        ),
        (
            "rand-0.8.5.json",
            "getrandom",
            {
                "cargo",
                "crate(rand) = 0.8.5",
                "(crate(rand_core/getrandom) >= 0.6.0 with crate(rand_core/getrandom) < 0.7.0~)",
            },
        ),
        (
            "rand-0.8.5.json",
            "libc",
            {
                "cargo",
                "crate(rand) = 0.8.5",
                "(crate(libc) >= 0.2.22 with crate(libc) < 0.3.0~)",
            },
        ),
        (
            "rand-0.8.5.json",
            "log",
            {
                "cargo",
                "crate(rand) = 0.8.5",
                "(crate(log/default) >= 0.4.4 with crate(log/default) < 0.5.0~)",
            },
        ),
        (
            "rand-0.8.5.json",
            "min_const_gen",
            {
                "cargo",
                "crate(rand) = 0.8.5",
            },
        ),
        (
            "rand-0.8.5.json",
            "nightly",
            {
                "cargo",
                "crate(rand) = 0.8.5",
            },
        ),
        (
            "rand-0.8.5.json",
            "packed_simd",
            {
                "cargo",
                "crate(rand) = 0.8.5",
                "(crate(packed_simd_2/default) >= 0.3.7 with crate(packed_simd_2/default) < 0.4.0~)",
                "(crate(packed_simd_2/into_bits) >= 0.3.7 with crate(packed_simd_2/into_bits) < 0.4.0~)",
            },
        ),
        (
            "rand-0.8.5.json",
            "rand_chacha",
            {
                "cargo",
                "crate(rand) = 0.8.5",
                "(crate(rand_chacha) >= 0.3.0 with crate(rand_chacha) < 0.4.0~)",
            },
        ),
        (
            "rand-0.8.5.json",
            "serde",
            {
                "cargo",
                "crate(rand) = 0.8.5",
                "(crate(serde/default) >= 1.0.103 with crate(serde/default) < 2.0.0~)",
                "(crate(serde/derive) >= 1.0.103 with crate(serde/derive) < 2.0.0~)",
            },
        ),
        (
            "rand-0.8.5.json",
            "serde1",
            {
                "cargo",
                "crate(rand) = 0.8.5",
                "crate(rand/serde) = 0.8.5",
                "(crate(rand_core/serde1) >= 0.6.0 with crate(rand_core/serde1) < 0.7.0~)",
            },
        ),
        (
            "rand-0.8.5.json",
            "simd_support",
            {
                "cargo",
                "crate(rand) = 0.8.5",
                "crate(rand/packed_simd) = 0.8.5",
            },
        ),
        (
            "rand-0.8.5.json",
            "small_rng",
            {
                "cargo",
                "crate(rand) = 0.8.5",
            },
        ),
        (
            "rand-0.8.5.json",
            "std",
            {
                "cargo",
                "crate(rand) = 0.8.5",
                "crate(rand/alloc) = 0.8.5",
                "crate(rand/getrandom) = 0.8.5",
                "crate(rand/libc) = 0.8.5",
                "(crate(rand_core/std) >= 0.6.0 with crate(rand_core/std) < 0.7.0~)",
                "(crate(rand_chacha) >= 0.3.0 with crate(rand_chacha) < 0.4.0~)",
                "(crate(rand_chacha/std) >= 0.3.0 with crate(rand_chacha/std) < 0.4.0~)",
            },
        ),
        (
            "rand-0.8.5.json",
            "std_rng",
            {
                "cargo",
                "crate(rand) = 0.8.5",
                "crate(rand/rand_chacha) = 0.8.5",
            },
        ),
        (
            "rand_core-0.6.4.json",
            "default",
            {
                "cargo",
                "crate(rand_core) = 0.6.4",
            },
        ),
        (
            "rand_core-0.6.4.json",
            "alloc",
            {
                "cargo",
                "crate(rand_core) = 0.6.4",
            },
        ),
        (
            "rand_core-0.6.4.json",
            "getrandom",
            {
                "cargo",
                "crate(rand_core) = 0.6.4",
                "(crate(getrandom/default) >= 0.2.0 with crate(getrandom/default) < 0.3.0~)",
            },
        ),
        (
            "rand_core-0.6.4.json",
            "serde",
            {
                "cargo",
                "crate(rand_core) = 0.6.4",
                "(crate(serde/default) >= 1.0.0 with crate(serde/default) < 2.0.0~)",
                "(crate(serde/derive) >= 1.0.0 with crate(serde/derive) < 2.0.0~)",
            },
        ),
        (
            "rand_core-0.6.4.json",
            "serde1",
            {
                "cargo",
                "crate(rand_core) = 0.6.4",
                "crate(rand_core/serde) = 0.6.4",
            },
        ),
        (
            "rand_core-0.6.4.json",
            "std",
            {
                "cargo",
                "crate(rand_core) = 0.6.4",
                "crate(rand_core/alloc) = 0.6.4",
                "crate(rand_core/getrandom) = 0.6.4",
                "(crate(getrandom/default) >= 0.2.0 with crate(getrandom/default) < 0.3.0~)",
                "(crate(getrandom/std) >= 0.2.0 with crate(getrandom/std) < 0.3.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "default",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/serde) = 1.28.0",
                "crate(rust_decimal/std) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "arbitrary",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(arbitrary) >= 1.0.0 with crate(arbitrary) < 2.0.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "borsh",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(borsh) >= 0.9.0 with crate(borsh) < 0.10.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "bytecheck",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(bytecheck) >= 0.6.0 with crate(bytecheck) < 0.7.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "byteorder",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(byteorder) >= 1.0.0 with crate(byteorder) < 2.0.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "bytes",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(bytes) >= 1.0.0 with crate(bytes) < 2.0.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "c-repr",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "db-diesel-mysql",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/db-diesel1-mysql) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "db-diesel-postgres",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/db-diesel1-postgres) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "db-diesel1-mysql",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/std) = 1.28.0",
                "(crate(diesel) >= 1.0.0 with crate(diesel) < 2.0.0~)",
                "(crate(diesel/mysql) >= 1.0.0 with crate(diesel/mysql) < 2.0.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "db-diesel1-postgres",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/std) = 1.28.0",
                "(crate(diesel) >= 1.0.0 with crate(diesel) < 2.0.0~)",
                "(crate(diesel/postgres) >= 1.0.0 with crate(diesel/postgres) < 2.0.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "db-diesel2-mysql",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/std) = 1.28.0",
                "(crate(diesel) >= 2.0.0 with crate(diesel) < 3.0.0~)",
                "(crate(diesel/mysql) >= 2.0.0 with crate(diesel/mysql) < 3.0.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "db-diesel2-postgres",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/std) = 1.28.0",
                "(crate(diesel) >= 2.0.0 with crate(diesel) < 3.0.0~)",
                "(crate(diesel/postgres) >= 2.0.0 with crate(diesel/postgres) < 3.0.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "db-postgres",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/byteorder) = 1.28.0",
                "crate(rust_decimal/bytes) = 1.28.0",
                "crate(rust_decimal/postgres) = 1.28.0",
                "crate(rust_decimal/std) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "db-tokio-postgres",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/byteorder) = 1.28.0",
                "crate(rust_decimal/bytes) = 1.28.0",
                "crate(rust_decimal/postgres) = 1.28.0",
                "crate(rust_decimal/std) = 1.28.0",
                "crate(rust_decimal/tokio-postgres) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "diesel1",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(diesel) >= 1.0.0 with crate(diesel) < 2.0.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "diesel2",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(diesel) >= 2.0.0 with crate(diesel) < 3.0.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "legacy-ops",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "maths",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "maths-nopanic",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/maths) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "postgres",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(postgres) >= 0.19.0 with crate(postgres) < 0.20.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "rand",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(rand) >= 0.8.0 with crate(rand) < 0.9.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "rkyv",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(rkyv) >= 0.7.0 with crate(rkyv) < 0.8.0~)",
                "(crate(rkyv/size_32) >= 0.7.0 with crate(rkyv/size_32) < 0.8.0~)",
                "(crate(rkyv/std) >= 0.7.0 with crate(rkyv/std) < 0.8.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "rkyv-safe",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/bytecheck) = 1.28.0",
                "(crate(rkyv) >= 0.7.0 with crate(rkyv) < 0.8.0~)",
                "(crate(rkyv/size_32) >= 0.7.0 with crate(rkyv/size_32) < 0.8.0~)",
                "(crate(rkyv/std) >= 0.7.0 with crate(rkyv/std) < 0.8.0~)",
                "(crate(rkyv/validation) >= 0.7.0 with crate(rkyv/validation) < 0.8.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "rocket",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(rocket) >= 0.5.0~rc.1 with crate(rocket) < 0.6.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "rocket-traits",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/rocket) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "rust-fuzz",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/arbitrary) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "serde-arbitrary-precision",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/serde-with-arbitrary-precision) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "serde-bincode",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/serde-str) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "serde-float",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/serde-with-float) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "serde-str",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/serde-with-str) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "serde-with-arbitrary-precision",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/serde) = 1.28.0",
                "(crate(serde_json) >= 1.0.0 with crate(serde_json) < 2.0.0~)",
                "(crate(serde_json/arbitrary_precision) >= 1.0.0 with crate(serde_json/arbitrary_precision) < 2.0.0~)",
                "(crate(serde_json/std) >= 1.0.0 with crate(serde_json/std) < 2.0.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "serde-with-float",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/serde) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "serde-with-str",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/serde) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "serde_json",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(serde_json) >= 1.0.0 with crate(serde_json) < 2.0.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "std",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(arrayvec/std) >= 0.7.0 with crate(arrayvec/std) < 0.8.0~)",
                "(crate(borsh) >= 0.9.0 with crate(borsh) < 0.10.0~)",
                "(crate(borsh/std) >= 0.9.0 with crate(borsh/std) < 0.10.0~)",
                "(crate(bytecheck) >= 0.6.0 with crate(bytecheck) < 0.7.0~)",
                "(crate(bytecheck/std) >= 0.6.0 with crate(bytecheck/std) < 0.7.0~)",
                "(crate(byteorder) >= 1.0.0 with crate(byteorder) < 2.0.0~)",
                "(crate(byteorder/std) >= 1.0.0 with crate(byteorder/std) < 2.0.0~)",
                "(crate(bytes) >= 1.0.0 with crate(bytes) < 2.0.0~)",
                "(crate(bytes/std) >= 1.0.0 with crate(bytes/std) < 2.0.0~)",
                "(crate(rand) >= 0.8.0 with crate(rand) < 0.9.0~)",
                "(crate(rand/std) >= 0.8.0 with crate(rand/std) < 0.9.0~)",
                "(crate(rkyv) >= 0.7.0 with crate(rkyv) < 0.8.0~)",
                "(crate(rkyv/size_32) >= 0.7.0 with crate(rkyv/size_32) < 0.8.0~)",
                "(crate(rkyv/std) >= 0.7.0 with crate(rkyv/std) < 0.8.0~)",
                "(crate(serde) >= 1.0.0 with crate(serde) < 2.0.0~)",
                "(crate(serde/std) >= 1.0.0 with crate(serde/std) < 2.0.0~)",
                "(crate(serde_json) >= 1.0.0 with crate(serde_json) < 2.0.0~)",
                "(crate(serde_json/std) >= 1.0.0 with crate(serde_json/std) < 2.0.0~)",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "tokio-pg",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "crate(rust_decimal/db-tokio-postgres) = 1.28.0",
            },
        ),
        (
            "rust_decimal-1.28.0.json",
            "tokio-postgres",
            {
                "cargo",
                "crate(rust_decimal) = 1.28.0",
                "(crate(tokio-postgres) >= 0.7.0 with crate(tokio-postgres) < 0.8.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "default",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "crate(rustix/std) = 0.36.8",
                "crate(rustix/use-libc-auxv) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "all-apis",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "crate(rustix/fs) = 0.36.8",
                "crate(rustix/io_uring) = 0.36.8",
                "crate(rustix/mm) = 0.36.8",
                "crate(rustix/net) = 0.36.8",
                "crate(rustix/param) = 0.36.8",
                "crate(rustix/process) = 0.36.8",
                "crate(rustix/procfs) = 0.36.8",
                "crate(rustix/rand) = 0.36.8",
                "crate(rustix/runtime) = 0.36.8",
                "crate(rustix/termios) = 0.36.8",
                "crate(rustix/thread) = 0.36.8",
                "crate(rustix/time) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "all-impls",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "crate(rustix/fs-err) = 0.36.8",
                "crate(rustix/os_pipe) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "alloc",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "(crate(rustc-std-workspace-alloc/default) >= 1.0.0 with crate(rustc-std-workspace-alloc/default) < 2.0.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "cc",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "(crate(cc/default) >= 1.0.68 with crate(cc/default) < 2.0.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "compiler_builtins",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "(crate(compiler_builtins/default) >= 0.1.49 with crate(compiler_builtins/default) < 0.2.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "core",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "(crate(rustc-std-workspace-core/default) >= 1.0.0 with crate(rustc-std-workspace-core/default) < 2.0.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "fs",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "fs-err",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "(crate(io-lifetimes) >= 1.0.0 with crate(io-lifetimes) < 2.0.0~)",
                "(crate(io-lifetimes/close) >= 1.0.0 with crate(io-lifetimes/close) < 2.0.0~)",
                "(crate(io-lifetimes/fs-err) >= 1.0.0 with crate(io-lifetimes/fs-err) < 2.0.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "io-lifetimes",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "(crate(io-lifetimes) >= 1.0.0 with crate(io-lifetimes) < 2.0.0~)",
                "(crate(io-lifetimes/close) >= 1.0.0 with crate(io-lifetimes/close) < 2.0.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "io_uring",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "crate(rustix/fs) = 0.36.8",
                "crate(rustix/net) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "itoa",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "(crate(itoa) >= 1.0.1 with crate(itoa) < 2.0.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "libc",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "(crate(libc/default) >= 0.2.133 with crate(libc/default) < 0.3.0~)",
                "(crate(libc/extra_traits) >= 0.2.133 with crate(libc/extra_traits) < 0.3.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "libc_errno",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "(crate(errno) >= 0.2.8 with crate(errno) < 0.3.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "mm",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "net",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "once_cell",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "(crate(once_cell/default) >= 1.5.2 with crate(once_cell/default) < 2.0.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "os_pipe",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "(crate(io-lifetimes) >= 1.0.0 with crate(io-lifetimes) < 2.0.0~)",
                "(crate(io-lifetimes/close) >= 1.0.0 with crate(io-lifetimes/close) < 2.0.0~)",
                "(crate(io-lifetimes/os_pipe) >= 1.0.0 with crate(io-lifetimes/os_pipe) < 2.0.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "param",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "crate(rustix/fs) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "process",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "procfs",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "crate(rustix/once_cell) = 0.36.8",
                "crate(rustix/itoa) = 0.36.8",
                "crate(rustix/fs) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "rand",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "runtime",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "rustc-dep-of-std",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "crate(rustix/core) = 0.36.8",
                "crate(rustix/alloc) = 0.36.8",
                "crate(rustix/compiler_builtins) = 0.36.8",
                "(crate(linux-raw-sys/rustc-dep-of-std) >= 0.1.2 with crate(linux-raw-sys/rustc-dep-of-std) < 0.2.0~)",
                "(crate(bitflags/rustc-dep-of-std) >= 1.2.1 with crate(bitflags/rustc-dep-of-std) < 2.0.0~)",
            },
        ),
        (
            "rustix-0.36.8.json",
            "std",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "crate(rustix/io-lifetimes) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "termios",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "thread",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "time",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "use-libc",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "crate(rustix/libc) = 0.36.8",
                "crate(rustix/libc_errno) = 0.36.8",
            },
        ),
        (
            "rustix-0.36.8.json",
            "use-libc-auxv",
            {
                "cargo",
                "crate(rustix) = 0.36.8",
                "crate(rustix/libc) = 0.36.8",
            },
        ),
        (
            "serde-1.0.152.json",
            "default",
            {
                "cargo",
                "crate(serde) = 1.0.152",
                "crate(serde/std) = 1.0.152",
            },
        ),
        (
            "serde-1.0.152.json",
            "alloc",
            {
                "cargo",
                "crate(serde) = 1.0.152",
            },
        ),
        (
            "serde-1.0.152.json",
            "derive",
            {
                "cargo",
                "crate(serde) = 1.0.152",
                "crate(serde/serde_derive) = 1.0.152",
            },
        ),
        (
            "serde-1.0.152.json",
            "rc",
            {
                "cargo",
                "crate(serde) = 1.0.152",
            },
        ),
        (
            "serde-1.0.152.json",
            "serde_derive",
            {
                "cargo",
                "crate(serde) = 1.0.152",
                "crate(serde_derive/default) = 1.0.152",
            },
        ),
        (
            "serde-1.0.152.json",
            "std",
            {
                "cargo",
                "crate(serde) = 1.0.152",
            },
        ),
        (
            "serde-1.0.152.json",
            "unstable",
            {
                "cargo",
                "crate(serde) = 1.0.152",
            },
        ),
        (
            "serde_derive-1.0.152.json",
            "default",
            {
                "cargo",
                "crate(serde_derive) = 1.0.152",
            },
        ),
        (
            "serde_derive-1.0.152.json",
            "deserialize_in_place",
            {
                "cargo",
                "crate(serde_derive) = 1.0.152",
            },
        ),
        (
            "sha1collisiondetection-0.3.1.json",
            "default",
            {
                "cargo",
                "crate(sha1collisiondetection) = 0.3.1",
                "crate(sha1collisiondetection/std) = 0.3.1",
                "crate(sha1collisiondetection/digest-trait) = 0.3.1",
            },
        ),
        (
            "sha1collisiondetection-0.3.1.json",
            "digest-trait",
            {
                "cargo",
                "crate(sha1collisiondetection) = 0.3.1",
                "crate(sha1collisiondetection/digest) = 0.3.1",
            },
        ),
        (
            "sha1collisiondetection-0.3.1.json",
            "oid",
            {
                "cargo",
                "crate(sha1collisiondetection) = 0.3.1",
                "crate(sha1collisiondetection/const-oid) = 0.3.1",
            },
        ),
        (
            "sha1collisiondetection-0.3.1.json",
            "std",
            {
                "cargo",
                "crate(sha1collisiondetection) = 0.3.1",
                "(crate(digest/default) >= 0.10.0 with crate(digest/default) < 0.11.0~)",
                "(crate(digest/std) >= 0.10.0 with crate(digest/std) < 0.11.0~)",
            },
        ),
        (
            "syn-1.0.107.json",
            "default",
            {
                "cargo",
                "crate(syn) = 1.0.107",
                "crate(syn/derive) = 1.0.107",
                "crate(syn/parsing) = 1.0.107",
                "crate(syn/printing) = 1.0.107",
                "crate(syn/clone-impls) = 1.0.107",
                "crate(syn/proc-macro) = 1.0.107",
            },
        ),
        (
            "syn-1.0.107.json",
            "clone-impls",
            {
                "cargo",
                "crate(syn) = 1.0.107",
            },
        ),
        (
            "syn-1.0.107.json",
            "derive",
            {
                "cargo",
                "crate(syn) = 1.0.107",
            },
        ),
        (
            "syn-1.0.107.json",
            "extra-traits",
            {
                "cargo",
                "crate(syn) = 1.0.107",
            },
        ),
        (
            "syn-1.0.107.json",
            "fold",
            {
                "cargo",
                "crate(syn) = 1.0.107",
            },
        ),
        (
            "syn-1.0.107.json",
            "full",
            {
                "cargo",
                "crate(syn) = 1.0.107",
            },
        ),
        (
            "syn-1.0.107.json",
            "parsing",
            {
                "cargo",
                "crate(syn) = 1.0.107",
            },
        ),
        (
            "syn-1.0.107.json",
            "printing",
            {
                "cargo",
                "crate(syn) = 1.0.107",
                "crate(syn/quote) = 1.0.107",
            },
        ),
        (
            "syn-1.0.107.json",
            "proc-macro",
            {
                "cargo",
                "crate(syn) = 1.0.107",
                "(crate(proc-macro2/proc-macro) >= 1.0.46 with crate(proc-macro2/proc-macro) < 2.0.0~)",
                "(crate(quote) >= 1.0.0 with crate(quote) < 2.0.0~)",
                "(crate(quote/proc-macro) >= 1.0.0 with crate(quote/proc-macro) < 2.0.0~)",
            },
        ),
        (
            "syn-1.0.107.json",
            "quote",
            {
                "cargo",
                "crate(syn) = 1.0.107",
                "(crate(quote) >= 1.0.0 with crate(quote) < 2.0.0~)",
            },
        ),
        (
            "syn-1.0.107.json",
            "test",
            {
                "cargo",
                "crate(syn) = 1.0.107",
            },
        ),
        (
            "syn-1.0.107.json",
            "visit",
            {
                "cargo",
                "crate(syn) = 1.0.107",
            },
        ),
        (
            "syn-1.0.107.json",
            "visit-mut",
            {
                "cargo",
                "crate(syn) = 1.0.107",
            },
        ),
        (
            "time-0.3.17.json",
            "default",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "crate(time/std) = 0.3.17",
            },
        ),
        (
            "time-0.3.17.json",
            "default",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "crate(time/std) = 0.3.17",
            },
        ),
        (
            "time-0.3.17.json",
            "alloc",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "(crate(serde) >= 1.0.126 with crate(serde) < 2.0.0~)",
                "(crate(serde/alloc) >= 1.0.126 with crate(serde/alloc) < 2.0.0~)",
            },
        ),
        (
            "time-0.3.17.json",
            "formatting",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "crate(time/std) = 0.3.17",
                "crate(time-macros/default) = 0.2.6",
                "crate(time-macros/formatting) = 0.2.6",
                "(crate(itoa/default) >= 1.0.1 with crate(itoa/default) < 2.0.0~)",
            },
        ),
        (
            "time-0.3.17.json",
            "large-dates",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "crate(time-macros/default) = 0.2.6",
                "crate(time-macros/large-dates) = 0.2.6",
            },
        ),
        (
            "time-0.3.17.json",
            "local-offset",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "crate(time/std) = 0.3.17",
                "(crate(libc/default) >= 0.2.98 with crate(libc/default) < 0.3.0~)",
                "(crate(num_threads/default) >= 0.1.2 with crate(num_threads/default) < 0.2.0~)",
            },
        ),
        (
            "time-0.3.17.json",
            "macros",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "crate(time-macros/default) = 0.2.6",
            },
        ),
        (
            "time-0.3.17.json",
            "parsing",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "crate(time-macros/default) = 0.2.6",
                "crate(time-macros/parsing) = 0.2.6",
            },
        ),
        (
            "time-0.3.17.json",
            "rand",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "(crate(rand) >= 0.8.4 with crate(rand) < 0.9.0~)",
            },
        ),
        (
            "time-0.3.17.json",
            "serde",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "crate(time-macros/default) = 0.2.6",
                "crate(time-macros/serde) = 0.2.6",
                "(crate(serde) >= 1.0.126 with crate(serde) < 2.0.0~)",
            },
        ),
        (
            "time-0.3.17.json",
            "serde-human-readable",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "crate(time/serde) = 0.3.17",
                "crate(time/formatting) = 0.3.17",
                "crate(time/parsing) = 0.3.17",
            },
        ),
        (
            "time-0.3.17.json",
            "serde-well-known",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "crate(time/serde) = 0.3.17",
                "crate(time/formatting) = 0.3.17",
                "crate(time/parsing) = 0.3.17",
            },
        ),
        (
            "time-0.3.17.json",
            "std",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "crate(time/alloc) = 0.3.17",
            },
        ),
        (
            "time-0.3.17.json",
            "wasm-bindgen",
            {
                "cargo",
                "crate(time) = 0.3.17",
                "(crate(js-sys/default) >= 0.3.58 with crate(js-sys/default) < 0.4.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "default",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            "bytes",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "(crate(bytes/default) >= 1.0.0 with crate(bytes/default) < 2.0.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "fs",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            "full",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "crate(tokio/fs) = 1.25.0",
                "crate(tokio/io-util) = 1.25.0",
                "crate(tokio/io-std) = 1.25.0",
                "crate(tokio/macros) = 1.25.0",
                "crate(tokio/net) = 1.25.0",
                "crate(tokio/parking_lot) = 1.25.0",
                "crate(tokio/process) = 1.25.0",
                "crate(tokio/rt) = 1.25.0",
                "crate(tokio/rt-multi-thread) = 1.25.0",
                "crate(tokio/signal) = 1.25.0",
                "crate(tokio/sync) = 1.25.0",
                "crate(tokio/time) = 1.25.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            "io-std",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            "io-util",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "crate(tokio/memchr) = 1.25.0",
                "crate(tokio/bytes) = 1.25.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            "libc",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "(crate(libc/default) >= 0.2.42 with crate(libc/default) < 0.3.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "macros",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "crate(tokio/tokio-macros) = 1.25.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            "memchr",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "(crate(memchr/default) >= 2.2.0 with crate(memchr/default) < 3.0.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "mio",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "(crate(mio/default) >= 0.8.4 with crate(mio/default) < 0.9.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "net",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "crate(tokio/libc) = 1.25.0",
                "crate(tokio/socket2) = 1.25.0",
                "(crate(mio/default) >= 0.8.4 with crate(mio/default) < 0.9.0~)",
                "(crate(mio/os-poll) >= 0.8.4 with crate(mio/os-poll) < 0.9.0~)",
                "(crate(mio/os-ext) >= 0.8.4 with crate(mio/os-ext) < 0.9.0~)",
                "(crate(mio/net) >= 0.8.4 with crate(mio/net) < 0.9.0~)",
                "(crate(windows-sys/default) >= 0.42.0 with crate(windows-sys/default) < 0.43.0~)",
                "(crate(windows-sys/Win32_Foundation) >= 0.42.0 with crate(windows-sys/Win32_Foundation) < 0.43.0~)",
                "(crate(windows-sys/Win32_Security) >= 0.42.0 with crate(windows-sys/Win32_Security) < 0.43.0~)",
                "(crate(windows-sys/Win32_Storage_FileSystem) >= 0.42.0 with crate(windows-sys/Win32_Storage_FileSystem) < 0.43.0~)",
                "(crate(windows-sys/Win32_System_Pipes) >= 0.42.0 with crate(windows-sys/Win32_System_Pipes) < 0.43.0~)",
                "(crate(windows-sys/Win32_System_SystemServices) >= 0.42.0 with crate(windows-sys/Win32_System_SystemServices) < 0.43.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "num_cpus",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "(crate(num_cpus/default) >= 1.8.0 with crate(num_cpus/default) < 2.0.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "parking_lot",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "(crate(parking_lot/default) >= 0.12.0 with crate(parking_lot/default) < 0.13.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "process",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "crate(tokio/bytes) = 1.25.0",
                "crate(tokio/libc) = 1.25.0",
                "crate(tokio/signal-hook-registry) = 1.25.0",
                "(crate(mio/default) >= 0.8.4 with crate(mio/default) < 0.9.0~)",
                "(crate(mio/os-poll) >= 0.8.4 with crate(mio/os-poll) < 0.9.0~)",
                "(crate(mio/os-ext) >= 0.8.4 with crate(mio/os-ext) < 0.9.0~)",
                "(crate(mio/net) >= 0.8.4 with crate(mio/net) < 0.9.0~)",
                "(crate(windows-sys/default) >= 0.42.0 with crate(windows-sys/default) < 0.43.0~)",
                "(crate(windows-sys/Win32_Foundation) >= 0.42.0 with crate(windows-sys/Win32_Foundation) < 0.43.0~)",
                "(crate(windows-sys/Win32_System_Threading) >= 0.42.0 with crate(windows-sys/Win32_System_Threading) < 0.43.0~)",
                "(crate(windows-sys/Win32_System_WindowsProgramming) >= 0.42.0 with crate(windows-sys/Win32_System_WindowsProgramming) < 0.43.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "rt",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            "rt-multi-thread",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "crate(tokio/rt) = 1.25.0",
                "crate(tokio/num_cpus) = 1.25.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            "signal",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "crate(tokio/libc) = 1.25.0",
                "crate(tokio/signal-hook-registry) = 1.25.0",
                "(crate(mio/default) >= 0.8.4 with crate(mio/default) < 0.9.0~)",
                "(crate(mio/os-poll) >= 0.8.4 with crate(mio/os-poll) < 0.9.0~)",
                "(crate(mio/os-ext) >= 0.8.4 with crate(mio/os-ext) < 0.9.0~)",
                "(crate(mio/net) >= 0.8.4 with crate(mio/net) < 0.9.0~)",
                "(crate(windows-sys/default) >= 0.42.0 with crate(windows-sys/default) < 0.43.0~)",
                "(crate(windows-sys/Win32_Foundation) >= 0.42.0 with crate(windows-sys/Win32_Foundation) < 0.43.0~)",
                "(crate(windows-sys/Win32_System_Console) >= 0.42.0 with crate(windows-sys/Win32_System_Console) < 0.43.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "signal-hook-registry",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "(crate(signal-hook-registry/default) >= 1.1.1 with crate(signal-hook-registry/default) < 2.0.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "socket2",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "(crate(socket2/default) >= 0.4.4 with crate(socket2/default) < 0.5.0~)",
                "(crate(socket2/all) >= 0.4.4 with crate(socket2/all) < 0.5.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "stats",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            "sync",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            "test-util",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "crate(tokio/rt) = 1.25.0",
                "crate(tokio/sync) = 1.25.0",
                "crate(tokio/time) = 1.25.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            "time",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
            },
        ),
        (
            "tokio-1.25.0.json",
            "tokio-macros",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "(crate(tokio-macros/default) >= 1.7.0 with crate(tokio-macros/default) < 2.0.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "tracing",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "(crate(tracing) >= 0.1.25 with crate(tracing) < 0.2.0~)",
                "(crate(tracing/std) >= 0.1.25 with crate(tracing/std) < 0.2.0~)",
            },
        ),
        (
            "tokio-1.25.0.json",
            "windows-sys",
            {
                "cargo",
                "crate(tokio) = 1.25.0",
                "(crate(windows-sys/default) >= 0.42.0 with crate(windows-sys/default) < 0.43.0~)",
            },
        ),
        (
            "unicode-xid-0.2.4.json",
            "default",
            {
                "cargo",
                "crate(unicode-xid) = 0.2.4",
            },
        ),
        (
            "unicode-xid-0.2.4.json",
            "bench",
            {
                "cargo",
                "crate(unicode-xid) = 0.2.4",
            },
        ),
        (
            "unicode-xid-0.2.4.json",
            "no_std",
            {
                "cargo",
                "crate(unicode-xid) = 0.2.4",
            },
        ),
        (
            "zbus-3.8.0.json",
            "default",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "crate(zbus/async-io) = 3.8.0",
            },
        ),
        (
            "zbus-3.8.0.json",
            "async-executor",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "(crate(async-executor/default) >= 1.5.0 with crate(async-executor/default) < 2.0.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "async-io",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "crate(zbus/async-executor) = 3.8.0",
                "crate(zbus/async-task) = 3.8.0",
                "crate(zbus/async-lock) = 3.8.0",
                "(crate(async-io/default) >= 1.12.0 with crate(async-io/default) < 2.0.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "async-lock",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "(crate(async-lock/default) >= 2.6.0 with crate(async-lock/default) < 3.0.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "async-task",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "(crate(async-task/default) >= 4.3.0 with crate(async-task/default) < 5.0.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "chrono",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "(crate(zvariant/chrono) >= 3.10.0 with crate(zvariant/chrono) < 4.0.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "gvariant",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "(crate(zvariant/gvariant) >= 3.10.0 with crate(zvariant/gvariant) < 4.0.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "lazy_static",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "(crate(lazy_static/default) >= 1.4.0 with crate(lazy_static/default) < 2.0.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "quick-xml",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "(crate(quick-xml/default) >= 0.27.1 with crate(quick-xml/default) < 0.28.0~)",
                "(crate(quick-xml/serialize) >= 0.27.1 with crate(quick-xml/serialize) < 0.28.0~)",
                "(crate(quick-xml/overlapped-lists) >= 0.27.1 with crate(quick-xml/overlapped-lists) < 0.28.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "serde-xml-rs",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "(crate(serde-xml-rs/default) >= 0.4.1 with crate(serde-xml-rs/default) < 0.5.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "time",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "(crate(zvariant/time) >= 3.10.0 with crate(zvariant/time) < 4.0.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "tokio",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "crate(zbus/lazy_static) = 3.8.0",
                "(crate(tokio/default) >= 1.21.2 with crate(tokio/default) < 2.0.0~)",
                "(crate(tokio/rt) >= 1.21.2 with crate(tokio/rt) < 2.0.0~)",
                "(crate(tokio/net) >= 1.21.2 with crate(tokio/net) < 2.0.0~)",
                "(crate(tokio/time) >= 1.21.2 with crate(tokio/time) < 2.0.0~)",
                "(crate(tokio/fs) >= 1.21.2 with crate(tokio/fs) < 2.0.0~)",
                "(crate(tokio/io-util) >= 1.21.2 with crate(tokio/io-util) < 2.0.0~)",
                "(crate(tokio/sync) >= 1.21.2 with crate(tokio/sync) < 2.0.0~)",
                "(crate(tokio/tracing) >= 1.21.2 with crate(tokio/tracing) < 2.0.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "tokio-vsock",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "crate(zbus/tokio) = 3.8.0",
                "(crate(tokio-vsock/default) >= 0.3.3 with crate(tokio-vsock/default) < 0.4.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "url",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "(crate(zvariant/url) >= 3.10.0 with crate(zvariant/url) < 4.0.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "uuid",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "(crate(zvariant/uuid) >= 3.10.0 with crate(zvariant/uuid) < 4.0.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "vsock",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "(crate(async-io/default) >= 1.12.0 with crate(async-io/default) < 2.0.0~)",
                "(crate(vsock/default) >= 0.3.0 with crate(vsock/default) < 0.4.0~)",
            },
        ),
        (
            "zbus-3.8.0.json",
            "windows-gdbus",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
            },
        ),
        (
            "zbus-3.8.0.json",
            "xml",
            {
                "cargo",
                "crate(zbus) = 3.8.0",
                "crate(zbus/serde-xml-rs) = 3.8.0",
            },
        ),
    ],
    ids=short_repr,
)
def test_requires_feature(filename: str, feature: str, expected: set[str]):
    metadata = load_metadata_from_resource(filename)
    assert _requires_feature(metadata.packages[0], feature) == expected



from cargo2rpm.utils import load_metadata_from_resource, short_repr

import pytest


@pytest.mark.parametrize(
    "filename,optional,expected",
    [
        ("ahash-0.8.3.json", False, {"cfg-if", "once_cell"}),
        ("ahash-0.8.3.json", True, {"atomic-polyfill", "const-random", "getrandom", "serde"}),
        ("aho-corasick-1.0.2.json", False, set()),
        ("aho-corasick-1.0.2.json", True, {"log", "memchr"}),
        ("assert_cmd-2.0.8.json", False, {"bstr", "doc-comment", "predicates", "predicates-core", "predicates-tree", "wait-timeout"}),
        ("assert_cmd-2.0.8.json", True, {"concolor", "yansi"}),
        ("assert_fs-1.0.10.json", False, {"doc-comment", "globwalk", "predicates", "predicates-core", "predicates-tree", "tempfile"}),
        ("assert_fs-1.0.10.json", True, {"concolor", "yansi"}),
        ("autocfg-1.1.0.json", False, set()),
        ("autocfg-1.1.0.json", True, set()),
        ("bstr-1.2.0.json", False, {"memchr"}),
        ("bstr-1.2.0.json", True, {"once_cell", "regex-automata", "serde"}),
        ("cfg-if-1.0.0.json", False, set()),
        ("cfg-if-1.0.0.json", True, {"compiler_builtins", "core"}),
        ("clap-4.1.4.json", False, {"bitflags", "clap_lex"}),
        (
            "clap-4.1.4.json",
            True,
            {"backtrace", "clap_derive", "is-terminal", "once_cell", "strsim", "termcolor", "terminal_size", "unicase", "unicode-width"},
        ),
        (
            "gstreamer-0.19.7.json",
            False,
            {
                "bitflags",
                "cfg-if",
                "ffi",
                "futures-channel",
                "futures-core",
                "futures-util",
                "glib",
                "libc",
                "muldiv",
                "num-integer",
                "num-rational",
                "once_cell",
                "opt-ops",
                "paste",
                "pretty-hex",
                "thiserror",
            },
        ),
        ("gstreamer-0.19.7.json", True, {"serde", "serde_bytes"}),
        ("human-panic-1.1.0.json", False, {"backtrace", "os_info", "serde", "serde_derive", "toml", "uuid"}),
        ("human-panic-1.1.0.json", True, {"concolor", "termcolor"}),
        (
            "hyperfine-1.15.0.json",
            False,
            {
                "anyhow",
                "atty",
                "clap",
                "colored",
                "csv",
                "indicatif",
                "rand",
                "rust_decimal",
                "serde",
                "serde_json",
                "shell-words",
                "statistical",
                "thiserror",
                "once_cell",
                "libc",
                "nix",
                "winapi",
            },
        ),
        ("hyperfine-1.15.0.json", True, set()),
        ("iri-string-0.7.0.json", False, set()),
        ("iri-string-0.7.0.json", True, {"memchr", "serde"}),
        ("libc-0.2.139.json", False, set()),
        ("libc-0.2.139.json", True, {"rustc-std-workspace-core"}),
        ("predicates-2.1.5.json", False, {"itertools", "predicates-core"}),
        ("predicates-2.1.5.json", True, {"concolor", "difflib", "float-cmp", "normalize-line-endings", "regex", "yansi"}),
        ("proc-macro2-1.0.50.json", False, {"unicode-ident"}),
        ("proc-macro2-1.0.50.json", True, set()),
        ("quote-1.0.23.json", False, {"proc-macro2"}),
        ("quote-1.0.23.json", True, set()),
        ("rand-0.8.5.json", False, {"rand_core"}),
        ("rand-0.8.5.json", True, {"log", "packed_simd", "rand_chacha", "serde", "libc"}),
        ("rand_core-0.6.4.json", False, set()),
        ("rand_core-0.6.4.json", True, {"getrandom", "serde"}),
        ("regex-1.8.4.json", False, {"regex-syntax"}),
        ("regex-1.8.4.json", True, {"aho-corasick", "memchr"}),
        ("regex-syntax-0.7.2.json", False, set()),
        ("regex-syntax-0.7.2.json", True, set()),
        ("rust_decimal-1.28.0.json", False, {"arrayvec", "num-traits"}),
        (
            "rust_decimal-1.28.0.json",
            True,
            {
                "arbitrary",
                "borsh",
                "bytecheck",
                "byteorder",
                "bytes",
                "diesel1",
                "diesel2",
                "postgres",
                "rand",
                "rkyv",
                "rocket",
                "serde",
                "serde_json",
                "tokio-postgres",
            },
        ),
        ("rustix-0.36.8.json", False, {"bitflags", "linux-raw-sys", "libc", "libc_errno", "windows-sys"}),
        ("rustix-0.36.8.json", True, {"alloc", "compiler_builtins", "core", "io-lifetimes", "itoa", "libc", "libc_errno", "once_cell"}),
        ("serde-1.0.152.json", False, set()),
        ("serde-1.0.152.json", True, {"serde_derive"}),
        ("serde_derive-1.0.152.json", False, {"proc-macro2", "quote", "syn"}),
        ("serde_derive-1.0.152.json", True, set()),
        ("sha1collisiondetection-0.3.1.json", False, {"generic-array"}),
        ("sha1collisiondetection-0.3.1.json", True, {"clap", "clap_mangen", "const-oid", "digest"}),
        ("syn-1.0.107.json", False, {"proc-macro2", "unicode-ident"}),
        ("syn-1.0.107.json", True, {"quote"}),
        ("time-0.3.17.json", False, {"time-core"}),
        ("time-0.3.17.json", True, {"itoa", "quickcheck", "rand", "serde", "time-macros", "js-sys", "libc", "num_threads"}),
        ("tokio-1.25.0.json", False, {"pin-project-lite", "windows-sys"}),
        (
            "tokio-1.25.0.json",
            True,
            {
                "bytes",
                "memchr",
                "mio",
                "num_cpus",
                "parking_lot",
                "tokio-macros",
                "socket2",
                "tracing",
                "libc",
                "signal-hook-registry",
                "windows-sys",
            },
        ),
        ("unicode-xid-0.2.4.json", False, set()),
        ("unicode-xid-0.2.4.json", True, set()),
        (
            "zbus-3.8.0.json",
            False,
            {
                "async-broadcast",
                "async-recursion",
                "async-trait",
                "byteorder",
                "derivative",
                "dirs",
                "enumflags2",
                "event-listener",
                "futures-core",
                "futures-sink",
                "futures-util",
                "hex",
                "nix",
                "once_cell",
                "ordered-stream",
                "rand",
                "serde",
                "serde_repr",
                "sha1",
                "static_assertions",
                "tracing",
                "zbus_macros",
                "zbus_names",
                "zvariant",
                "uds_windows",
                "winapi",
            },
        ),
        (
            "zbus-3.8.0.json",
            True,
            {
                "async-executor",
                "async-io",
                "async-lock",
                "async-task",
                "lazy_static",
                "quick-xml",
                "serde-xml-rs",
                "tokio",
                "tokio-vsock",
                "vsock",
            },
        ),
        (
            "zoxide-0.9.0.json",
            False,
            {"anyhow", "askama", "bincode", "clap", "dirs", "dunce", "fastrand", "glob", "ouroboros", "serde", "nix", "which"},
        ),
        ("zoxide-0.9.0.json", True, set()),
    ],
    ids=short_repr,
)
def test_package_get_normal_dependencies(filename: str, optional: bool, expected: set[str]):
    data = load_metadata_from_resource(filename)
    assert set(data.packages[0].get_normal_dependencies(optional).keys()) == expected


@pytest.mark.parametrize(
    "filename,expected",
    [
        (
            "ahash-0.8.3.json",
            {
                "criterion",
                "fnv",
                "fxhash",
                "hashbrown",
                "hex",
                "no-panic",
                "rand",
                "seahash",
                "serde_json",
            },
        ),
        ("aho-corasick-1.0.2.json", {"doc-comment"}),
        ("assert_cmd-2.0.8.json", {"escargot"}),
        ("assert_fs-1.0.10.json", set()),
        ("autocfg-1.1.0.json", set()),
        ("bstr-1.2.0.json", {"quickcheck", "ucd-parse", "unicode-segmentation"}),
        ("cfg-if-1.0.0.json", set()),
        ("clap-4.1.4.json", {"humantime", "rustversion", "shlex", "snapbox", "static_assertions", "trybuild", "trycmd", "unic-emoji-char"}),
        ("gstreamer-0.19.7.json", {"futures-executor", "gir-format-check", "ron", "serde_json"}),
        ("human-panic-1.1.0.json", set()),
        ("hyperfine-1.15.0.json", {"approx", "assert_cmd", "predicates", "tempfile"}),
        ("iri-string-0.7.0.json", {"serde_test"}),
        ("libc-0.2.139.json", set()),
        ("predicates-2.1.5.json", {"predicates-tree"}),
        ("proc-macro2-1.0.50.json", {"quote"}),
        ("quote-1.0.23.json", {"rustversion", "trybuild"}),
        ("rand-0.8.5.json", {"bincode", "rand_pcg"}),
        ("rand_core-0.6.4.json", set()),
        ("regex-1.8.4.json", {"lazy_static", "quickcheck", "rand"}),
        ("regex-syntax-0.7.2.json", set()),
        (
            "rust_decimal-1.28.0.json",
            {"bincode", "bytes", "criterion", "csv", "futures", "rand", "serde", "serde_json", "tokio", "version-sync"},
        ),
        (
            "rustix-0.36.8.json",
            {"flate2", "io-lifetimes", "libc", "libc_errno", "memoffset", "serial_test", "tempfile", "criterion", "ctor"},
        ),
        ("serde-1.0.152.json", {"serde_derive"}),
        ("serde_derive-1.0.152.json", {"serde"}),
        ("sha1collisiondetection-0.3.1.json", {"getrandom", "hex-literal", "sha1"}),
        (
            "syn-1.0.107.json",
            {
                "anyhow",
                "automod",
                "flate2",
                "insta",
                "rayon",
                "ref-cast",
                "regex",
                "reqwest",
                "syn-test-suite",
                "tar",
                "termcolor",
                "walkdir",
            },
        ),
        ("time-0.3.17.json", {"quickcheck_macros", "rand", "serde", "serde_json", "serde_test", "time-macros", "trybuild", "criterion"}),
        (
            "tokio-1.25.0.json",
            {
                "async-stream",
                "futures",
                "mockall",
                "tempfile",
                "tokio-stream",
                "tokio-test",
                "wasm-bindgen-test",
                "loom",
                "rand",
                "proptest",
                "socket2",
                "mio-aio",
                "libc",
                "nix",
                "ntapi",
            },
        ),
        ("unicode-xid-0.2.4.json", {"criterion"}),
        ("zbus-3.8.0.json", {"async-std", "doc-comment", "futures-util", "ntest", "tempfile", "test-log", "tokio", "tracing-subscriber"}),
        ("zoxide-0.9.0.json", {"assert_cmd", "rstest", "rstest_reuse", "tempfile"}),
    ],
    ids=short_repr,
)
def test_package_get_dev_dependencies(filename: str, expected: set[str]):
    data = load_metadata_from_resource(filename)
    assert set(data.packages[0].get_dev_dependencies().keys()) == expected


@pytest.mark.parametrize(
    "filename,optional,expected",
    [
        ("ahash-0.8.3.json", False, {"version_check"}),
        ("ahash-0.8.3.json", True, set()),
        ("aho-corasick-1.0.2.json", False, set()),
        ("aho-corasick-1.0.2.json", True, set()),
        ("assert_cmd-2.0.8.json", False, set()),
        ("assert_cmd-2.0.8.json", True, set()),
        ("assert_fs-1.0.10.json", False, set()),
        ("assert_fs-1.0.10.json", True, set()),
        ("autocfg-1.1.0.json", True, set()),
        ("autocfg-1.1.0.json", False, set()),
        ("bstr-1.2.0.json", False, set()),
        ("bstr-1.2.0.json", True, set()),
        ("cfg-if-1.0.0.json", False, set()),
        ("cfg-if-1.0.0.json", True, set()),
        ("clap-4.1.4.json", False, set()),
        ("clap-4.1.4.json", True, set()),
        ("gstreamer-0.19.7.json", False, set()),
        ("gstreamer-0.19.7.json", True, set()),
        ("human-panic-1.1.0.json", False, set()),
        ("human-panic-1.1.0.json", True, set()),
        ("hyperfine-1.15.0.json", False, {"atty", "clap", "clap_complete"}),
        ("hyperfine-1.15.0.json", True, set()),
        ("iri-string-0.7.0.json", False, set()),
        ("iri-string-0.7.0.json", True, set()),
        ("libc-0.2.139.json", False, set()),
        ("libc-0.2.139.json", True, set()),
        ("predicates-2.1.5.json", False, set()),
        ("predicates-2.1.5.json", True, set()),
        ("proc-macro2-1.0.50.json", False, set()),
        ("proc-macro2-1.0.50.json", True, set()),
        ("quote-1.0.23.json", False, set()),
        ("quote-1.0.23.json", True, set()),
        ("rand-0.8.5.json", False, set()),
        ("rand-0.8.5.json", True, set()),
        ("rand_core-0.6.4.json", False, set()),
        ("rand_core-0.6.4.json", True, set()),
        ("regex-1.8.4.json", False, set()),
        ("regex-1.8.4.json", True, set()),
        ("regex-syntax-0.7.2.json", False, set()),
        ("regex-syntax-0.7.2.json", True, set()),
        ("rust_decimal-1.28.0.json", False, set()),
        ("rust_decimal-1.28.0.json", True, set()),
        ("rustix-0.36.8.json", False, set()),
        ("rustix-0.36.8.json", True, {"cc"}),
        ("serde-1.0.152.json", False, set()),
        ("serde-1.0.152.json", True, set()),
        ("serde_derive-1.0.152.json", False, set()),
        ("serde_derive-1.0.152.json", True, set()),
        ("sha1collisiondetection-0.3.1.json", False, set()),
        ("sha1collisiondetection-0.3.1.json", True, {"clap", "clap_mangen"}),
        ("syn-1.0.107.json", False, set()),
        ("syn-1.0.107.json", True, set()),
        ("time-0.3.17.json", False, set()),
        ("time-0.3.17.json", True, set()),
        ("tokio-1.25.0.json", False, {"autocfg"}),
        ("tokio-1.25.0.json", True, set()),
        ("unicode-xid-0.2.4.json", False, set()),
        ("unicode-xid-0.2.4.json", True, set()),
        ("zbus-3.8.0.json", False, set()),
        ("zbus-3.8.0.json", True, set()),
        ("zoxide-0.9.0.json", False, {"clap", "clap_complete", "clap_complete_fig"}),
        ("zoxide-0.9.0.json", True, set()),
    ],
    ids=short_repr,
)
def test_package_get_build_dependencies(filename: str, optional: bool, expected: set[str]):
    data = load_metadata_from_resource(filename)
    assert set(data.packages[0].get_build_dependencies(optional).keys()) == expected



import os
import subprocess
from typing import Optional

from cargo2rpm import CARGO
from cargo2rpm.metadata import FeatureFlags


def license_breakdown(flags: FeatureFlags, package: Optional[str] = None) -> set[str]:
    cmd = [
        CARGO,
        "tree",
        "-Zavoid-dev-deps",
        "--package={package}" if package else "--workspace",
        "--offline",
        "--edges=no-build,no-dev,no-proc-macro",
        "--no-dedupe",
        "--target=all",
        "--prefix=none",
        "--format",
        "{l}: {p}",
    ]
    if flags.all_features:
        cmd.append("--all-features")
    if flags.no_default_features:
        cmd.append("--no-default-features")
    if flags.features:
        cmd.append("--features={}".format(",".join(flags.features)))

    ret = subprocess.run(cmd, capture_output=True)
    ret.check_returncode()

    cwd = os.getcwd()
    lines = ret.stdout.decode().splitlines()
    items = {line.replace(f" ({cwd})", "").replace(" / ", "/").replace("/", " OR ") for line in lines}

    return items


def license_summary(flags: FeatureFlags, package: Optional[str] = None) -> set[str]:
    cmd = [
        CARGO,
        "tree",
        "-Zavoid-dev-deps",
        "--package={package}" if package else "--workspace",
        "--offline",
        "--edges=no-build,no-dev,no-proc-macro",
        "--no-dedupe",
        "--target=all",
        "--prefix=none",
        "--format",
        "# {l}",
    ]
    if flags.all_features:
        cmd.append("--all-features")
    if flags.no_default_features:
        cmd.append("--no-default-features")
    if flags.features:
        cmd.append("--features={}".format(",".join(flags.features)))

    ret = subprocess.run(cmd, capture_output=True)
    ret.check_returncode()

    lines = ret.stdout.decode().splitlines()
    items = {line.replace(" / ", "/").replace("/", " OR ") for line in lines}

    return items



"""
This module is a partial Python port of the "semver" crate:
<https://docs.rs/semver>
"""

from enum import Enum
import itertools
import re
from typing import Optional


VERSION_REGEX = re.compile(
    r"""
    ^
    (?P<major>0|[1-9]\d*)
    \.(?P<minor>0|[1-9]\d*)
    \.(?P<patch>0|[1-9]\d*)
    (?:-(?P<pre>(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?
    (?:\+(?P<build>[0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$
    """,
    re.VERBOSE,
)


VERSION_REQ_REGEX = re.compile(
    r"""
    ^
    (?P<op>=|>|>=|<|<=|~|\^)?
    (?P<major>0|[1-9]\d*)
    (\.(?P<minor>\*|0|[1-9]\d*))?
    (\.(?P<patch>\*|0|[1-9]\d*))?
    (-(?P<pre>(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?
    $
    """,
    re.VERBOSE,
)


class PreRelease:
    """
    Pre-release part of a Version.
    """

    def __init__(self, parts: list[str | int]):
        self.parts: list[str | int] = parts

    @staticmethod
    def parse(prerelease: str) -> "PreRelease":
        parts: list[str | int] = [int(part) if part.isdecimal() else part for part in prerelease.split(".")]

        return PreRelease(parts)

    def __str__(self):
        return ".".join([str(part) for part in self.parts])

    def __repr__(self):
        return repr(str(self))

    def __eq__(self, other):
        if not isinstance(other, PreRelease):
            return False  # pragma nocover

        return self.parts == other.parts

    def __lt__(self, other):
        """
        Algorithm for determining precedence between pre-releases based on the semver.org spec.
        """

        if not isinstance(other, PreRelease):
            return False  # pragma nocover

        for lpart, rpart in itertools.zip_longest(self.parts, other.parts):
            match lpart, rpart:
                # all previous parts were equal and the left pre-release has more parts
                case l, None:
                    return False

                # all previous parts were equal and the right pre-release has more parts
                case None, r:
                    return True

                # compare nonempty parts depending on value type
                case l, r:
                    match isinstance(l, int), isinstance(r, int):
                        # both parts are numbers: compare numerically
                        case True, True:
                            if l < r:
                                return True
                            elif l == r:
                                continue
                            else:
                                return False

                        # both parts are strings: compare lexicographically
                        case False, False:
                            if l < r:
                                return True
                            elif l == r:
                                continue
                            else:
                                return False

                        # number and string: string takes precedence
                        case True, False:
                            return True

                        # string and number: string takes precedence
                        case False, True:
                            return False

                        case _:  # pragma nocover
                            raise RuntimeError("Unreachable: This should never happen.")

                case _:  # pragma nocover
                    raise RuntimeError("Unreachable: This should never happen.")

        # both pre-releases have equal numbers of parts and all pairs of parts are equal
        else:
            return False

    def __le__(self, other):
        if not isinstance(other, PreRelease):
            return False  # pragma nocover

        return (self == other) or (self < other)

    def __gt__(self, other):
        if not isinstance(other, PreRelease):
            return False  # pragma nocover

        return other < self

    def __ge__(self, other):
        if not isinstance(other, PreRelease):
            return False  # pragma nocover

        return (self == other) or (self > other)


class Version:
    """
    Version that adheres to the "semantic versioning" format.
    """

    def __init__(self, major: int, minor: int, patch: int, pre: Optional[str] = None, build: Optional[str] = None):
        self.major: int = major
        self.minor: int = minor
        self.patch: int = patch
        self.pre: Optional[str] = pre
        self.build: Optional[str] = build

    @staticmethod
    def parse(version: str) -> "Version":
        """
        Parses a version string and return a `Version` object.
        Raises a `ValueError` if the string does not match the expected format.
        """

        match = VERSION_REGEX.match(version)
        if not match:
            raise ValueError(f"Invalid version: {version!r}")

        matches = match.groupdict()

        major_str = matches["major"]
        minor_str = matches["minor"]
        patch_str = matches["patch"]
        pre = matches["pre"]
        build = matches["build"]

        major = int(major_str)
        minor = int(minor_str)
        patch = int(patch_str)

        return Version(major, minor, patch, pre, build)

    def __str__(self):
        s = f"{self.major}.{self.minor}.{self.patch}"
        if self.pre:
            s += f"-{self.pre}"
        if self.build:
            s += f"+{self.build}"
        return s

    def __repr__(self):
        return repr(str(self))

    def __eq__(self, other):
        if not isinstance(other, Version):
            return False  # pragma nocover

        return (self.major == other.major) and (self.minor == other.minor) and (self.patch == other.patch) and (self.pre == other.pre)

    def __lt__(self, other):
        """
        Algorithm for determining precedence between versions based on the semver.org spec.
        """

        if not isinstance(other, Version):
            return False  # pragma nocover

        if self.major < other.major:
            return True
        if self.major > other.major:
            return False

        # major versions match
        if self.minor < other.minor:
            return True
        if self.minor > other.minor:
            return False

        # minor versions match
        if self.patch < other.patch:
            return True
        if self.patch > other.patch:
            return False

        # patch versions match
        match self.pre, other.pre:
            case None, None:
                return False
            case spre, None:
                return True
            case None, opre:
                return False
            case spre, opre:
                return PreRelease.parse(spre) < PreRelease.parse(opre)
            case _:  # pragma nocover
                raise RuntimeError("Unreachable: This should never happen.")

    def __le__(self, other):
        if not isinstance(other, Version):
            return False  # pragma nocover

        return (self == other) or (self < other)

    def __gt__(self, other):
        if not isinstance(other, Version):
            return False  # pragma nocover

        return other < self

    def __ge__(self, other):
        if not isinstance(other, Version):
            return False  # pragma nocover

        return (self == other) or (self > other)

    def to_rpm(self) -> str:
        """
        Formats the `Version` object as an equivalent RPM version string.
        Characters that are invalid in RPM versions are replaced ("-" -> "_")

        Build metadata (the optional `Version.build` attribute) is dropped, so
        the conversion is not lossless for versions where this attribute is not
        `None`. However, build metadata is not intended to be part of the
        version (and is not even considered when doing version comparison), so
        dropping it when converting to the RPM version format is correct.
        """

        s = f"{self.major}.{self.minor}.{self.patch}"
        if self.pre:
            s += f"~{self.pre.replace('-', '_')}"
        return s

    @staticmethod
    def from_rpm(version: str) -> "Version":
        """
        Parses an RPM version string and return the equivalent `Version`.
        Characters that are invalid in SemVer format are replaced ("_" -> "-").

        This method performs the inverse of `Version.to_rpm`.
        """

        return Version.parse(version.replace("~", "-").replace("_", "-"))


class Op(Enum):
    """
    Version requirement operator.

    This class enumerates all operators that are considered valid for specifying
    the required version of a dependency in cargo.
    """

    EXACT = "="
    GREATER = ">"
    GREATER_EQ = ">="
    LESS = "<"
    LESS_EQ = "<="
    TILDE = "~"
    CARET = "^"
    WILDCARD = "*"

    def __repr__(self):
        return self.value  # pragma nocover


class Comparator:
    """
    Partial version requirement.

    A `Comparator` consists of an operator (`Op`) and a (partial) semantic
    version and is used to define a requirement that a version can be matched
    against.
    """

    def __init__(self, op: Op, major: int, minor: Optional[int], patch: Optional[int], pre: Optional[str]):
        self.op: Op = op
        self.major: int = major
        self.minor: Optional[int] = minor
        self.patch: Optional[int] = patch
        self.pre: Optional[str] = pre

    def __str__(self):
        if self.op == Op.WILDCARD:
            if self.minor is not None:
                return f"{self.major}.{self.minor}.*"
            else:
                return f"{self.major}.*"

        op = self.op.value
        if self.pre is not None:
            return f"{op}{self.major}.{self.minor}.{self.patch}-{self.pre}"
        if self.patch is not None:
            return f"{op}{self.major}.{self.minor}.{self.patch}"
        if self.minor is not None:
            return f"{op}{self.major}.{self.minor}"
        return f"{op}{self.major}"

    def __repr__(self):
        return repr(str(self))

    def __eq__(self, other):
        if not isinstance(other, Comparator):
            return False  # pragma nocover

        # naive equality check: does not take equivalence into account
        return (
            self.op == other.op
            and self.major == other.major
            and self.minor == other.minor
            and self.patch == other.patch
            and self.pre == other.pre
        )

    def __contains__(self, item):
        if not isinstance(item, Version):
            return False  # pragma nocover

        match self.op:
            case Op.EXACT:
                return item == self._as_version()
            case Op.GREATER:
                return item > self._as_version()
            case Op.GREATER_EQ:
                return item >= self._as_version()
            case Op.LESS:
                return item < self._as_version()
            case Op.LESS_EQ:
                return item <= self._as_version()
            case Op.TILDE:
                return item in VersionReq(self.normalize())
            case Op.CARET:
                return item in VersionReq(self.normalize())
            case Op.WILDCARD:
                return item in VersionReq(self.normalize())
            case _:  # pragma nocover
                raise ValueError(f"Unknown operator: {self.op} (this should never happen)")

    def _as_version(self) -> Version:
        return Version(self.major, self.minor or 0, self.patch or 0, self.pre)

    @staticmethod
    def parse(comparator: str) -> "Comparator":
        """
        Parses a single version requirement string and return a `Comparator`.
        Raises a `ValueError` if the string does not match the expected format.
        """

        match = VERSION_REQ_REGEX.match(comparator)
        if not match:
            raise ValueError(f"Invalid version requirement: {comparator!r}")

        matches = match.groupdict()

        op_str = matches["op"]
        major_str = matches["major"]
        minor_str = matches["minor"]
        patch_str = matches["patch"]
        pre = matches["pre"]

        # if patch is present, minor needs to be present as well
        if minor_str is None and patch_str is not None:
            raise ValueError(f"Invalid version requirement: {comparator!r}")  # pragma nocover

        # if patch is not wildcard, then minor cannot be wildcard
        if minor_str is not None and patch_str is not None and minor_str == "*" and patch_str != "*":
            raise ValueError(f"Invalid wildcard requirement: {comparator!r}.")

        # if pre-release is specified, then minor and patch must be present
        if pre and (minor_str is None or patch_str is None):
            raise ValueError(f"Invalid pre-release requirement (minor / patch version missing): {comparator!r}")

        # normalize wildcard specifiers
        if minor_str is not None and minor_str == "*":
            op_str = "*"
            minor_str = None
        if patch_str is not None and patch_str == "*":
            op_str = "*"
            patch_str = None

        # fall back to default CARET ("^") operator if not specified
        op = Op(op_str) if op_str is not None else Op.CARET
        major = int(major_str)
        minor = int(minor_str) if minor_str is not None else None
        patch = int(patch_str) if patch_str is not None else None

        return Comparator(op, major, minor, patch, pre)

    def normalize(self) -> list["Comparator"]:
        """
        Normalizes this comparator into a list of equivalent comparators which
        only use the ">=", ">", "<", "<=", and "=" operators.

        This is based on the documentation of the semver crate, which is used
        by cargo: <https://docs.rs/semver/1.0.16/semver/enum.Op.html>

        This normalized version requirement can be formatted as a valid RPM
        dependency string. Other operators (i.e. "^", "~", and "*") are not
        supported by RPM.
        """

        comparators = []

        match self.op:
            case Op.EXACT:
                match (self.minor, self.patch):
                    case (None, None):
                        comparators.append(Comparator(Op.GREATER_EQ, self.major, 0, 0, None))
                        comparators.append(Comparator(Op.LESS, self.major + 1, 0, 0, None))
                    case (minor, None):
                        comparators.append(Comparator(Op.GREATER_EQ, self.major, minor, 0, None))
                        comparators.append(Comparator(Op.LESS, self.major, minor + 1, 0, None))  # type: ignore
                    case (minor, patch):
                        comparators.append(Comparator(Op.EXACT, self.major, minor, patch, self.pre))
                    case _:  # pragma nocover
                        raise RuntimeError("Unreachable: This should never happen.")

            case Op.GREATER:
                match (self.minor, self.patch):
                    case (None, None):
                        comparators.append(Comparator(Op.GREATER_EQ, self.major + 1, 0, 0, None))
                    case (minor, None):
                        comparators.append(Comparator(Op.GREATER_EQ, self.major, minor + 1, 0, None))  # type: ignore
                    case (minor, patch):
                        comparators.append(Comparator(Op.GREATER, self.major, minor, patch, self.pre))
                    case _:  # pragma nocover
                        raise RuntimeError("Unreachable: This should never happen.")

            case Op.GREATER_EQ:
                match (self.minor, self.patch):
                    case (None, None):
                        comparators.append(Comparator(Op.GREATER_EQ, self.major, 0, 0, None))
                    case (minor, None):
                        comparators.append(Comparator(Op.GREATER_EQ, self.major, minor, 0, None))
                    case (minor, patch):
                        comparators.append(Comparator(Op.GREATER_EQ, self.major, minor, patch, self.pre))
                    case _:  # pragma nocover
                        raise RuntimeError("Unreachable: This should never happen.")

            case Op.LESS:
                match (self.minor, self.patch):
                    case (None, None):
                        comparators.append(Comparator(Op.LESS, self.major, 0, 0, None))
                    case (minor, None):
                        comparators.append(Comparator(Op.LESS, self.major, minor, 0, None))
                    case (minor, patch):
                        comparators.append(Comparator(Op.LESS, self.major, minor, patch, self.pre))
                    case _:  # pragma nocover
                        raise RuntimeError("Unreachable: This should never happen.")

            case Op.LESS_EQ:
                match (self.minor, self.patch):
                    case (None, None):
                        comparators.append(Comparator(Op.LESS, self.major + 1, 0, 0, None))
                    case (minor, None):
                        comparators.append(Comparator(Op.LESS, self.major, minor + 1, 0, None))  # type: ignore
                    case (minor, patch):
                        comparators.append(Comparator(Op.LESS_EQ, self.major, minor, patch, self.pre))
                    case _:  # pragma nocover
                        raise RuntimeError("Unreachable: This should never happen.")

            case Op.TILDE:
                match (self.minor, self.patch):
                    case (None, None):
                        comparators.extend(Comparator(Op.EXACT, self.major, None, None, None).normalize())
                    case (minor, None):
                        comparators.extend(Comparator(Op.EXACT, self.major, minor, None, None).normalize())
                    case (minor, patch):
                        comparators.append(Comparator(Op.GREATER_EQ, self.major, minor, patch, self.pre))
                        comparators.append(Comparator(Op.LESS, self.major, minor + 1, 0, None))  # type: ignore
                    case _:  # pragma nocover
                        raise RuntimeError("Unreachable: This should never happen.")

            case Op.CARET:
                match (self.major, self.minor, self.patch):
                    case (major, None, None):
                        comparators.extend(Comparator(Op.EXACT, major, None, None, None).normalize())
                    case (0, 0, None):
                        comparators.extend(Comparator(Op.EXACT, 0, 0, None, None).normalize())
                    case (major, minor, None):
                        comparators.extend(Comparator(Op.CARET, major, minor, 0, None).normalize())
                    case (0, 0, patch):
                        comparators.extend(Comparator(Op.EXACT, 0, 0, patch, self.pre).normalize())
                    case (0, minor, patch):
                        comparators.append(Comparator(Op.GREATER_EQ, 0, minor, patch, self.pre))
                        comparators.append(Comparator(Op.LESS, 0, minor + 1, 0, None))  # type: ignore
                    case (major, minor, patch):
                        comparators.append(Comparator(Op.GREATER_EQ, major, minor, patch, self.pre))
                        comparators.append(Comparator(Op.LESS, major + 1, 0, 0, None))
                    case _:  # pragma nocover
                        raise RuntimeError("Unreachable: This should never happen.")

            case Op.WILDCARD:
                match (self.minor, self.patch):
                    case (None, None):
                        comparators.extend(Comparator(Op.EXACT, self.major, None, None, None).normalize())
                    case (minor, None):
                        comparators.extend(Comparator(Op.EXACT, self.major, minor, None, None).normalize())
                    case _:  # pragma nocover
                        raise RuntimeError("Unreachable: This should never happen.")

            case _:  # pragma nocover
                raise ValueError(f"Unknown operator: {self.op} (this should never happen)")

        return comparators

    def to_rpm(self, crate: str, feature: Optional[str]) -> str:
        """
        Formats the `Comparator` object as an equivalent RPM dependency.
        Raises a `ValueError` if the comparator cannot be converted into a valid
        RPM dependency (for example, if it was not normalized or uses an
        unsupported operator).
        """

        if self.normalize() != [self]:
            raise ValueError("Cannot format non-normalized comparators in RPM syntax.")  # pragma nocover

        if feature is None:
            feature_str = ""
        else:
            feature_str = f"/{feature}"

        version_str = f"{self.major}.{self.minor}.{self.patch}"

        if self.pre is None:
            pre_str = ""
            pre_str_less = "~"
        else:
            pre = self.pre.replace("-", "_")
            pre_str = f"~{pre}"
            pre_str_less = f"~{pre}"

        match self.op:
            case Op.EXACT:
                return f"crate({crate}{feature_str}) = {version_str}{pre_str}"
            case Op.GREATER:
                return f"crate({crate}{feature_str}) > {version_str}{pre_str}"
            case Op.GREATER_EQ:
                return f"crate({crate}{feature_str}) >= {version_str}{pre_str}"
            case Op.LESS:
                return f"crate({crate}{feature_str}) < {version_str}{pre_str_less}"
            case Op.LESS_EQ:
                return f"crate({crate}{feature_str}) <= {version_str}{pre_str}"
            case _:  # pragma nocover
                raise ValueError(f"Unsupported operator for RPM syntax formatting: {self.op}")


class VersionReq:
    """
    Version requirement.

    A `VersionReq` consists of a - possibly empty -list of a `Comparators`.

    An empty list represents the "no requirements" case (i.e. any version
    matches this requirement).

    For a version to match a version requirement with a non-empty list of
    comparators, it must match with all comparators in the list.
    """

    def __init__(self, comparators: list[Comparator]):
        self.comparators = comparators

    def __str__(self):
        if not self.comparators:
            return "*"

        return ",".join(str(comparator) for comparator in self.comparators)

    def __repr__(self):
        return repr(str(self))

    def __eq__(self, other):
        if not isinstance(other, VersionReq):
            return False  # pragma nocover

        return self.comparators == other.comparators

    def __contains__(self, item):
        if not isinstance(item, Version):
            return False  # pragma nocover

        normalized = self.normalize().comparators
        return all(item in comparator for comparator in normalized)

    @staticmethod
    def parse(req: str) -> "VersionReq":
        """
        Parses a version requirement string and return a `VersionReq`.
        Raises a `ValueError` if the string does not match the expected format.
        """

        if not req:
            raise ValueError("Invalid version requirement (empty string).")

        if req == "*":
            return VersionReq([])

        reqs = req.replace(" ", "").split(",")
        comparators = [Comparator.parse(req) for req in reqs]

        return VersionReq(comparators)

    def normalize(self) -> "VersionReq":
        """
        Normalizes this version requirement into an equivalent requirement with
        comparators that only use ">=", ">", "<", "<=", and "=" operators.
        Other operators (i.e. "^", "~", and "*") are not supported by RPM.
        """

        comparators = []
        for comparator in self.comparators:
            comparators.extend(comparator.normalize())

        return VersionReq(comparators)

    def to_rpm(self, crate: str, feature: Optional[str]) -> str:
        """
        Formats the `VersionReq` object as an equivalent RPM dependency string.

        Raises a `ValueError` if the requirement cannot be converted into a
        valid RPM dependency - for example, if normalizing the comparators in
        this requirement results in a list of comparators with three or more
        items, which cannot easily be represented as RPM dependencies.
        """

        comparators = self.normalize().comparators

        if len(comparators) == 0:
            if feature is None:
                feature_str = ""
            else:
                feature_str = f"/{feature}"
            return f"crate({crate}{feature_str})"

        if len(comparators) == 1:
            return comparators[0].to_rpm(crate, feature)

        if len(comparators) == 2:
            return f"({comparators[0].to_rpm(crate, feature)} with {comparators[1].to_rpm(crate, feature)})"

        # len(comparators) > 2:
        raise ValueError("Using more than 2 comparators is not supported by RPM.")



from typing import Optional

from cargo2rpm.cli import get_args, get_feature_from_subpackage
from cargo2rpm.rpm import InvalidFeatureError
from cargo2rpm.utils import short_repr

import pytest


@pytest.mark.parametrize(
    "args,expected",
    [
        (
            [],
            {
                "action": None,
                "path": None,
            },
        ),
        (
            ["buildrequires"],
            {
                "action": "buildrequires",
                "all_features": False,
                "features": None,
                "no_default_features": False,
                "path": None,
                "with_check": False,
            },
        ),
        (
            ["buildrequires", "-t", "-a"],
            {
                "action": "buildrequires",
                "all_features": True,
                "features": None,
                "no_default_features": False,
                "path": None,
                "with_check": True,
            },
        ),
        (
            ["requires"],
            {
                "action": "requires",
                "feature": None,
                "path": None,
                "subpackage": False,
            },
        ),
        (
            ["requires", "-f", "default"],
            {
                "action": "requires",
                "feature": "default",
                "path": None,
                "subpackage": False,
            },
        ),
        (
            ["requires", "-s", "-f", "rust-ahash+default-devel"],
            {
                "action": "requires",
                "feature": "rust-ahash+default-devel",
                "path": None,
                "subpackage": True,
            },
        ),
        (
            ["provides"],
            {
                "action": "provides",
                "feature": None,
                "path": None,
                "subpackage": False,
            },
        ),
        (
            ["provides", "-f", "default"],
            {
                "action": "provides",
                "feature": "default",
                "path": None,
                "subpackage": False,
            },
        ),
        (
            ["provides", "-s", "-f", "rust-ahash+default-devel"],
            {
                "action": "provides",
                "feature": "rust-ahash+default-devel",
                "path": None,
                "subpackage": True,
            },
        ),
    ],
    ids=short_repr,
)
def test_get_args(args, expected):
    assert vars(get_args(args)) == expected


@pytest.mark.parametrize(
    "subpackage,expected",
    [
        ("rust-ahash-devel", None),
        ("rust-ahash+default-devel", "default"),
        ("rust-cxx-devel", None),
        ("rust-cxx+c++14-devel", "c++14"),
        ("rust-cxx+c++17-devel", "c++17"),
        ("rust-cxx+c++20-devel", "c++20"),
    ],
    ids=short_repr,
)
def test_get_feature_from_subpackage(subpackage: str, expected: Optional[str]):
    assert get_feature_from_subpackage(subpackage) == expected


@pytest.mark.parametrize(
    "invalid,expected",
    [
        ("firefox", "Invalid subpackage name (missing 'rust-' prefix)"),
        ("rust-packaging", "Invalid subpackage name (missing '-devel' suffix)"),
        ("rust-+invalid-devel", "Invalid subpackage name (crate name cannot be empty or contain '+' characters)"),
        ("rust-+foo+invalid-devel", "Invalid subpackage name (crate name cannot be empty or contain '+' characters)"),
    ],
    ids=short_repr,
)
def test_get_feature_from_subpackage_fail(invalid: str, expected: str):
    with pytest.raises(InvalidFeatureError) as exc:
        get_feature_from_subpackage(invalid)
    assert expected in str(exc.value)



from cargo2rpm.rpm import devel_subpackage_names
from cargo2rpm.utils import load_metadata_from_resource, short_repr

import pytest


@pytest.mark.parametrize(
    "filename,expected",
    [
        (
            "ahash-0.8.3.json",
            {"default", "atomic-polyfill", "compile-time-rng", "const-random", "getrandom", "no-rng", "runtime-rng", "serde", "std"},
        ),
        (
            "assert_cmd-2.0.8.json",
            {"default", "color", "color-auto"},
        ),
        (
            "assert_fs-1.0.10.json",
            {"default", "color", "color-auto"},
        ),
        (
            "autocfg-1.1.0.json",
            {"default"},
        ),
        (
            "bstr-1.2.0.json",
            {"default", "alloc", "serde", "std", "unicode"},
        ),
        (
            "cfg-if-1.0.0.json",
            {"default", "compiler_builtins", "core", "rustc-dep-of-std"},
        ),
        (
            "clap-4.1.4.json",
            {
                "default",
                "cargo",
                "color",
                "debug",
                "deprecated",
                "derive",
                "env",
                "error-context",
                "help",
                "std",
                "string",
                "suggestions",
                "unicode",
                "unstable-doc",
                "unstable-grouped",
                "unstable-replace",
                "unstable-v5",
                "usage",
                "wrap_help",
            },
        ),
        (
            "gstreamer-0.19.7.json",
            {"default", "dox", "serde", "serde_bytes", "v1_16", "v1_18", "v1_20", "v1_22"},
        ),
        (
            "human-panic-1.1.0.json",
            {"default", "color", "nightly"},
        ),
        (
            "hyperfine-1.15.0.json",
            set(),
        ),
        (
            "libc-0.2.139.json",
            {"default", "align", "const-extern-fn", "extra_traits", "rustc-dep-of-std", "rustc-std-workspace-core", "std", "use_std"},
        ),
        (
            "predicates-2.1.5.json",
            {"default", "color", "color-auto", "diff", "float-cmp", "normalize-line-endings", "regex", "unstable"},
        ),
        (
            "proc-macro2-1.0.50.json",
            {"default", "nightly", "proc-macro", "span-locations"},
        ),
        (
            "quote-1.0.23.json",
            {"default", "proc-macro"},
        ),
        (
            "rand-0.8.5.json",
            {
                "default",
                "alloc",
                "getrandom",
                "libc",
                "log",
                "min_const_gen",
                "nightly",
                "packed_simd",
                "rand_chacha",
                "serde",
                "serde1",
                "simd_support",
                "small_rng",
                "std",
                "std_rng",
            },
        ),
        (
            "rand_core-0.6.4.json",
            {"default", "alloc", "getrandom", "serde", "serde1", "std"},
        ),
        (
            "rpm-sequoia-1.2.0.json",
            set(),
        ),
        (
            "rust_decimal-1.28.0.json",
            {
                "default",
                "arbitrary",
                "borsh",
                "bytecheck",
                "byteorder",
                "bytes",
                "c-repr",
                "db-diesel-mysql",
                "db-diesel-postgres",
                "db-diesel1-mysql",
                "db-diesel1-postgres",
                "db-diesel2-mysql",
                "db-diesel2-postgres",
                "db-postgres",
                "db-tokio-postgres",
                "diesel1",
                "diesel2",
                "legacy-ops",
                "maths",
                "maths-nopanic",
                "postgres",
                "rand",
                "rkyv",
                "rkyv-safe",
                "rocket",
                "rocket-traits",
                "rust-fuzz",
                "serde",
                "serde-arbitrary-precision",
                "serde-bincode",
                "serde-float",
                "serde-str",
                "serde-with-arbitrary-precision",
                "serde-with-float",
                "serde-with-str",
                "serde_json",
                "std",
                "tokio-pg",
                "tokio-postgres",
            },
        ),
        (
            "rustix-0.36.8.json",
            {
                "default",
                "all-apis",
                "all-impls",
                "alloc",
                "cc",
                "compiler_builtins",
                "core",
                "fs",
                "fs-err",
                "io-lifetimes",
                "io_uring",
                "itoa",
                "libc",
                "libc_errno",
                "mm",
                "net",
                "once_cell",
                "os_pipe",
                "param",
                "process",
                "procfs",
                "rand",
                "runtime",
                "rustc-dep-of-std",
                "std",
                "termios",
                "thread",
                "time",
                "use-libc",
                "use-libc-auxv",
            },
        ),
        (
            "serde-1.0.152.json",
            {"default", "alloc", "derive", "rc", "serde_derive", "std", "unstable"},
        ),
        (
            "serde_derive-1.0.152.json",
            {"default", "deserialize_in_place"},
        ),
        (
            "sha1collisiondetection-0.3.1.json",
            {"default", "digest-trait", "oid", "std", "clap", "clap_mangen", "const-oid", "digest"},
        ),
        (
            "syn-1.0.107.json",
            {
                "default",
                "clone-impls",
                "derive",
                "extra-traits",
                "fold",
                "full",
                "parsing",
                "printing",
                "proc-macro",
                "quote",
                "test",
                "visit",
                "visit-mut",
            },
        ),
        (
            "time-0.3.17.json",
            {
                "default",
                "alloc",
                "formatting",
                "large-dates",
                "local-offset",
                "macros",
                "parsing",
                "quickcheck",
                "rand",
                "serde",
                "serde-human-readable",
                "serde-well-known",
                "std",
                "wasm-bindgen",
            },
        ),
        (
            "tokio-1.25.0.json",
            {
                "default",
                "bytes",
                "fs",
                "full",
                "io-std",
                "io-util",
                "libc",
                "macros",
                "memchr",
                "mio",
                "net",
                "num_cpus",
                "parking_lot",
                "process",
                "rt",
                "rt-multi-thread",
                "signal",
                "signal-hook-registry",
                "socket2",
                "stats",
                "sync",
                "test-util",
                "time",
                "tokio-macros",
                "tracing",
                "windows-sys",
            },
        ),
        (
            "unicode-xid-0.2.4.json",
            {"default", "bench", "no_std"},
        ),
        (
            "zbus-3.8.0.json",
            {
                "default",
                "async-executor",
                "async-io",
                "async-lock",
                "async-task",
                "chrono",
                "gvariant",
                "lazy_static",
                "quick-xml",
                "serde-xml-rs",
                "time",
                "tokio",
                "tokio-vsock",
                "url",
                "uuid",
                "vsock",
                "windows-gdbus",
                "xml",
            },
        ),
        (
            "zoxide-0.9.0.json",
            set(),
        ),
    ],
    ids=short_repr,
)
def test_devel_subpackage_names(filename: str, expected: set[str]):
    metadata = load_metadata_from_resource(filename)
    assert devel_subpackage_names(metadata.packages[0]) == expected
